
from filelike.wrappers import Head

import unittest
from StringIO import StringIO

class Test_Head(unittest.TestCase):
    """Testcases for the Head wrapper class."""
    
    def setUp(self):
        self.intext = "Guido van Rossum\n is a space\n alien."
        self.infile = StringIO(self.intext)
        self.outfile = StringIO()

    def tearDown(self):
        pass

    def test_ReadHeadBytes(self):
        """Test reading bytes from head of a file."""
        hf = Head(self.infile,"r",bytes=10)
        txt = hf.read()
        self.assertEquals(len(txt),10)
        self.assertEquals(txt,self.intext[:10])
    
    def test_ReadHeadLongBytes(self):
        """Test reading entirety of head of file."""
        hf = Head(self.infile,"r",bytes=1000)
        txt = hf.read()
        self.assertEquals(txt,self.intext)
    
    def test_ReadHeadLines(self):
        """Test reading lines from head of file."""
        hf = Head(self.infile,"r",lines=2)
        txt = hf.read()
        self.assertEquals(txt.count("\n"),2)
        self.assertEquals(txt,"\n".join(self.intext.split("\n")[:2])+"\n")

    def test_ReadHeadLinesExact(self):
        """Test reading exact number of lines from head of file."""
        hf = Head(self.infile,"r",lines=3)
        txt = hf.read()
        self.assertEquals(txt.count("\n"),2)
        self.assertEquals(txt,self.intext)

    def test_ReadHeadLongLines(self):
        """Test reading all lines from head of file."""
        hf = Head(self.infile,"r",lines=200)
        txt = hf.read()
        self.assertEquals(txt,self.intext)
        
    def test_ReadBytesOverLines(self):
        """Test reading limited by bytes, not lines"""
        hf = Head(self.infile,"r",bytes=5,lines=2)
        txt = hf.read()
        self.assertEquals(len(txt),5)
        self.assertEquals(txt,self.intext[:5])
        
    def test_ReadLinesOverBytes(self):
        """Test reading limited by lines, not bytes"""
        hf = Head(self.infile,"r",bytes=500,lines=1)
        txt = hf.read()
        self.assertEquals(txt.count("\n"),1)
        self.assertEquals(txt,self.intext.split("\n")[0]+"\n")

    def test_WriteHeadBytes(self):
        """Test writing bytes to head of a file."""
        hf = Head(self.outfile,"w",bytes=10)
        hf.write(self.intext)
        self.assertEquals(len(self.outfile.getvalue()),10)
        self.assertEquals(self.outfile.getvalue(),self.intext[:10])
    
    def test_WriteHeadLongBytes(self):
        """Test writing entirety of head of file."""
        hf = Head(self.outfile,"w",bytes=1000)
        hf.write(self.intext)
        self.assertEquals(self.outfile.getvalue(),self.intext)
    
    def test_WriteHeadLines(self):
        """Test writing lines to head of file."""
        hf = Head(self.outfile,"w",lines=2)
        hf.write(self.intext)
        self.assertEquals(self.outfile.getvalue().count("\n"),2)
        self.assertEquals(self.outfile.getvalue(),"\n".join(self.intext.split("\n")[:2])+"\n")

    def test_WriteHeadLongLines(self):
        """Test writing all lines to head of file."""
        hf = Head(self.outfile,"w",lines=200)
        hf.write(self.intext)
        self.assertEquals(self.outfile.getvalue(),self.intext)
        
    def test_WriteBytesOverLines(self):
        """Test writing limited by bytes, not lines"""
        hf = Head(self.outfile,"w",bytes=5,lines=2)
        hf.write(self.intext)
        txt = self.outfile.getvalue()
        self.assertEquals(len(txt),5)
        self.assertEquals(txt,self.intext[:5])
        
    def test_writeLinesOverBytes(self):
        """Test writing limited by lines, not bytes"""
        hf = Head(self.outfile,"w",bytes=500,lines=1)
        hf.write(self.intext)
        txt = self.outfile.getvalue()
        self.assertEquals(txt.count("\n"),1)
        self.assertEquals(txt,self.intext.split("\n")[0]+"\n")


def testsuite():
    suite = unittest.TestSuite()
    from filelike.wrappers import unix
    suite.addTest(unittest.makeSuite(Test_Head))
    return suite





from filelike.wrappers import *
from filelike import tests

import os
import tempfile
import unittest
from StringIO import StringIO


class Test_FileWrapper(tests.Test_ReadWriteSeek):
    """Testcases for FileWrapper base class."""
    
    def makeFile(self,contents,mode):
        s = StringIO(contents)
        f = FileWrapper(s,mode)
        def getvalue():
            return s.getvalue()
        f.getvalue = getvalue
        return f


class Test_OpenerDecoders(unittest.TestCase):
    """Testcases for the filelike.Opener decoder functions."""
    
    def setUp(self):
        import tempfile
        fd, self.tfilename = tempfile.mkstemp()
        os.close(fd)

    def tearDown(self):
        os.unlink(self.tfilename)

    def test_LocalFile(self):
        """Test opening a simple local file."""
        f = open(self.tfilename,"w")
        f.write("contents")
        f.close()
        f = filelike.open(self.tfilename,"r")
        self.assertEquals(f.name,self.tfilename)
        self.assertEquals(f.read(),"contents")
        f.close()
    
    def test_RemoteBzFile(self):
        """Test opening a remote BZ2 file."""
        f = filelike.open("http://www.rfk.id.au/static/test.txt.bz2","r-")
        self.assertEquals(f.read(),"contents goes here if you please.\n\n")




# filelike/wrappers/debug.py
#
# Copyright (C) 2009, Ryan Kelly
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
"""

    filelike.wrappers.debug:  debug accesses to a file

For the moment, just prints to stdout.
    
""" 

import filelike
from filelike.wrappers import FileWrapper

from StringIO import StringIO


class Debug(FileWrapper):
    """Class for debugging activity on a file.
    
    Currently it just logs activity to stdout, it will probably do
    more sophisticated things in the future.
    """

    def _debug(self,action,*args):
        args = [repr(a) for a in args]
        print "<%s : %s>  %s" % (self.label,action," | ".join(args))
    
    def __init__(self,fileobj,label="",mode=None):
        self.label = label
        self._debug("INIT",fileobj,mode)
        super(Debug,self).__init__(fileobj,mode)
    
    def _read(self,size=-1):
        self._debug("READING",size)
        data = self._fileobj.read(size)
        self._debug("READ",data)
        if data == "":
            return None
        return data
        
    def _write(self,data,flushing=False):
        self._debug("WRITING",data,flushing)
        self._fileobj.write(data)
        self._debug("WROTE")

    def _seek(self,offset,whence):
        self._debug("SEEKING",offset,whence)
        self._fileobj.seek(offset,whence)
        self._debug("SEEKED")

    def _tell(self):
        self._debug("TELLING")
        pos = self._fileobj.tell()
        self._debug("TELLED",pos)
        return pos

    def _truncate(self,size):
        self._debug("TRUNCATING",size)
        self._fileobj.truncate(size)
        self._debug("TRUNCATED")
        return pos

    def flush(self):
        self._debug("FLUSHING")
        self._fileobj.flush()
        self._debug("FLUSHED")

    def close(self):
        self._debug("CLOSING")
        self._fileobj.flush()
        self._debug("CLOSED")
 





from filelike.wrappers import Decrypt, Encrypt
from filelike import tests, NotSeekableError
from filelike.wrappers.tests.test_buffer import def_getvalue_maybe_buffered
import unittest
from StringIO import StringIO


class Test_Encrypt(tests.Test_ReadWriteSeek):
    """Testcases for the Encrypt wrapper class"""
    
    contents = "\x11,\xe3Nq\x8cDY\xdfT\xe2pA\xfa\xad\xc9s\x88\xf3,\xc0j\xd8\xa8\xca\xe7\xe2I\xd15w\x1d\xfe\x92\xd7\xca\xc9\xb5r\xec"
    plaintext = "Guido van Rossum is a space alien." + "\0"*6

    def makeFile(self,contents,mode):
        if len(contents) % self.cipher.block_size != 0:
            raise ValueError("content must be multiple of blocksize.")
        s = StringIO(self.cipher.decrypt(contents))
        f = Encrypt(s,self.cipher,mode=mode)
        f.getvalue = def_getvalue_maybe_buffered(f,s,self.cipher.encrypt)
        return f
        
    def setUp(self):
        from Crypto.Cipher import DES
        self.cipher = DES.new('abcdefgh',DES.MODE_ECB)
        super(Test_Encrypt,self).setUp()

    def test_append(self):
        method = super(Test_Encrypt,self).test_append
        self.assertRaises(NotSeekableError,method)


class Test_EncryptFB(tests.Test_ReadWriteSeek):
    """Testcases for the Encrypt wrapper class, using a feedback cipher"""
    
    contents = "\xc9\xa3b\x18\xeb\xe8\xbe3\x84\x9a,\x025\x13\xb0\xb7It\x90@a\xb1\xc2\x13\x04_6c\x19\x0b\xf2\xcd\x0eD\xfb?\xf5\xbb\xad\xc8"
    plaintext = "Guido van Rossum is a space alien." + "\0"*6

    def makeFile(self,contents,mode):
        IV = self.cipher.IV
        if len(contents) % self.cipher.block_size != 0:
            raise ValueError("content must be multiple of blocksize.")
        s = StringIO(self.cipher.decrypt(contents))
        self.cipher.IV = IV
        f = Encrypt(s,self.cipher,mode=mode)
        def trans(val):
            IV = self.cipher.IV
            self.cipher.IV = "12345678"
            val = self.cipher.encrypt(val)
            self.cipher.IV = IV
            return val
        f.getvalue = def_getvalue_maybe_buffered(f,s,trans)
        return f
        
    def setUp(self):
        from Crypto.Cipher import DES
        self.cipher = DES.new('abcdefgh',DES.MODE_CBC,"12345678")
        super(Test_EncryptFB,self).setUp()

    def test_append(self):
        method = super(Test_EncryptFB,self).test_append
        self.assertRaises(NotSeekableError,method)


class Test_Decrypt(tests.Test_ReadWriteSeek):
    """Testcases for the Decrypt wrapper class"""
    
    ciphertext = "\x11,\xe3Nq\x8cDY\xdfT\xe2pA\xfa\xad\xc9s\x88\xf3,\xc0j\xd8\xa8\xca\xe7\xe2I\xd15w\x1d\xfe\x92\xd7\xca\xc9\xb5r\xec"
    contents = "Guido van Rossum is a space alien." + "\0"*6

    def makeFile(self,contents,mode):
        if len(contents) % self.cipher.block_size != 0:
            raise ValueError("content must be multiple of blocksize.")
        s = StringIO(self.cipher.encrypt(contents))
        f = Decrypt(s,self.cipher,mode=mode)
        f.getvalue = def_getvalue_maybe_buffered(f,s,self.cipher.decrypt)
        return f
        
    def setUp(self):
        from Crypto.Cipher import DES
        self.cipher = DES.new('abcdefgh',DES.MODE_ECB)
        super(Test_Decrypt,self).setUp()

    def test_append(self):
        method = super(Test_Decrypt,self).test_append
        self.assertRaises(NotSeekableError,method)


class Test_DecryptFB(tests.Test_ReadWriteSeek):
    """Testcases for the Decrypt wrapper class, using a feedback cipher"""
    
    ciphertext = "\xc9\xa3b\x18\xeb\xe8\xbe3\x84\x9a,\x025\x13\xb0\xb7It\x90@a\xb1\xc2\x13\x04_6c\x19\x0b\xf2\xcd\x0eD\xfb?\xf5\xbb\xad\xc8"
    contents = "Guido van Rossum is a space alien." + "\0"*6

    def makeFile(self,contents,mode):
        IV = self.cipher.IV
        if len(contents) % self.cipher.block_size != 0:
            raise ValueError("content must be multiple of blocksize.")
        s = StringIO(self.cipher.encrypt(contents))
        self.cipher.IV = IV
        f = Decrypt(s,self.cipher,mode=mode)
        def trans(val):
            IV = self.cipher.IV
            self.cipher.IV = "12345678"
            val = self.cipher.decrypt(val)
            self.cipher.IV = IV
            return val
        f.getvalue = def_getvalue_maybe_buffered(f,s,trans)
        return f
        
    def setUp(self):
        from Crypto.Cipher import DES
        self.cipher = DES.new('abcdefgh',DES.MODE_CBC,"12345678")
        super(Test_DecryptFB,self).setUp()

    def test_append(self):
        method = super(Test_DecryptFB,self).test_append
        self.assertRaises(NotSeekableError,method)





# filelike/wrappers/translate.py
#
# Copyright (C) 2006-2009, Ryan Kelly
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
"""

    filelike.wrappers.translate:  pass file contents through translation func
    
This module provides the filelike wrapper 'Translate', which passes file
data through a translation function as it is read/written.  The default mode
of operation presumes a streaming translation function, which makes operations
such as seeking, or switching between reads and writes, quite expensive.

For more efficient operation the class 'BytewiseTranslate' is provided, which
assumes that the translation takes place on a byte-by-byte basis and can 
therefore optimise these operations.

""" 

import filelike
from filelike.wrappers import FileWrapper, Debug


class Translate(FileWrapper):
    """Class implementing some translation on a file's contents.
    
    This class wraps a file-like object in another file-like object,
    applying a given function to translate the file's contents as it is
    read or written.
    
    The translating function must accept a string as its only argument,
    and return a transformed string representing the updated file contents.
    If the function is symmetric than a single function may be specified;
    to use separate functions for reading and writing, provide the two
    keyword arguments 'rfunc' and 'wfunc'.

    If the transform needs to be flushed when reading/writing is finished, it
    should provide a flush() method that returns either None, or any data
    remaining to be read/written.  If it needs to be reset after flushing,
    it should provide a reset() method.

    If the translation function operates on a byte-by-byte basis and
    does not buffer any data, consider using the 'BytewiseTranslate'
    class instead; the efficiency of several operations can be improved
    dramatically given such properties of the translation function.
    """

    _append_requires_overwrite = True
    
    def __init__(self,fileobj,rfunc=None,wfunc=None,mode=None):
        """Translate file wrapper constructor.

        'fileobj' must be the file-like object whose contents are to be
        transformed, and 'rfunc' and 'wfunc' the callable objects that will
        transform the file's contents.
        """
        if mode is None:
            mode = getattr(fileobj,"mode","r+")
        # rfunc must be provided for readable files
        if self._check_mode("r-",mode):
            if rfunc is None:
                raise ValueError("Must provide 'rfunc' for readable files")
        # wfunc should be given for writable files, but we default to
        # using rfunc if it is specified.
        if self._check_mode("w-",mode):
            if wfunc is None:
                if rfunc is None:
                    raise ValueError("Must provide 'wfunc' for writable files")
                wfunc = rfunc
        self._rfunc = self._normalise_func(rfunc)
        self._wfunc = self._normalise_func(wfunc)
        self._pos = 0
        self._read_eof = False
        super(Translate,self).__init__(fileobj,mode)

    def _normalise_func(self,func):
        """Adjust a function to support flush() and reset() methods.

        This avoids the need to constantly check for these methods
        before invoking them.
        """
        def mknoop():
            def noop():
                return None
            return noop
        if func is None:
            func = mknoop()
        if not hasattr(func,"flush"):
            func.flush = mknoop()
        if not hasattr(func,"reset"):
            func.reset = mknoop()
        return func

    def flush(self):
        # TODO: this should read-and-write the rest of the data in the file
        data = self._wfunc.flush()
        if data is not None:
            self._fileobj.write(data)
        super(Translate,self).flush()
        if not self._closing:
            if "-" not in self.mode:
                self.seek(self.tell())
            else:
                if hasattr(self._rfunc,"reset"):
                    self._rfunc.reset()
                if hasattr(self._wfunc,"reset"):
                    self._wfunc.reset()

    def _read(self,sizehint=-1):
        if self._read_eof:
            return None
        data = self._fileobj.read(sizehint)
        if data == "":
            self._read_eof = True
            tData = self._rfunc.flush()
            if tData is None:
                return tData
        else:
            tData = self._rfunc(data)
        self._pos += len(tData)
        return tData
    
    def _write(self,data,flushing=False):
        """Write the given data to the file."""
        self._pos += len(data)
        wData = self._wfunc(data)
        self._fileobj.write(wData)
 
    def _tell(self):
        return self._pos

    def _seek(self,offset,whence):
        #  For generic translation functions, we can't do much more than
        #  go back to the beginning.  See BytewiseTranslate for a much
        #  more efficient seek().
        if whence > 0 or offset > 0:
            raise NotImplementedError
        self._fileobj.seek(0,0)
        self._pos = 0
        self._read_eof = False
        if hasattr(self._rfunc,"reset"):
            self._rfunc.reset()
        if hasattr(self._wfunc,"reset"):
            self._wfunc.reset()

    def _truncate(self,size):
        #  For generic translation functions, we can only sensibly truncate
        #  to zero bytes.  See BytewiseTranslate for truncation to any size.
        if size != 0:
            msg = "Translate wrapper can only be truncated to zero size"
            raise IOError(msg)
        self._seek(0,0)
        self._fileobj.truncate(0)
        

 
class BytewiseTranslate(FileWrapper):
    """Class implementing a bytewise translation on a file's contents.
    
    This class wraps a file-like object in another file-like object,
    applying a given function to translate the file's contents as it is
    read or written.  It could be used, for example, to encrypt a file
    as it's being written.
    
    The translating function must accept a string as its only argument,
    and return a transformed string representing the updated file contents.
    Since this is a bytewise translation, the returned string must be of
    the same length.  The translation function may not buffer any data.
    
    If a single function is provided it is used for both reads and writes.
    To use separate functions, provide the keyword arguments 'wfunc' and
    'rfunc'.
    """

    def __init__(self,fileobj,func=None,mode=None,rfunc=None,wfunc=None):
        """BytewiseTranslate file wrapper constructor.

        'fileobj' must be the file-like object whose contents are to be
        transformed, and 'func' the callable that will transform the
        contents.  'mode' should be one of "r" or "w" to indicate whether
        reading or writing is desired.  If omitted it is determined from
        'fileobj' where possible, otherwise it defaults to "r".
        
        If separate reading/writing translations are required, the
        keyword arguments 'rfunc' and 'wfunc' can be used in place of
        'func'.
        """
        super(BytewiseTranslate,self).__init__(fileobj,mode)
        if func is not None:
            if rfunc is not None:
                raise ValueError("Cannot specify both <func> and <rfunc>")
            if wfunc is not None:
                raise ValueError("Cannot specify both <func> and <wfunc>")
            self._rfunc = func
            self._wfunc = func
        else:
            if self._check_mode("r-"):
                if rfunc is None:
                    raise ValueError("Must provide <rfunc> for readable files")
            if self._check_mode("w-"):
                if wfunc is None:
                    raise ValueError("Must provide <wfunc> for writable files")
            self._rfunc = rfunc
            self._wfunc = wfunc
            
    def _read(self,sizehint=-1):
        """Read approximately <sizehint> bytes from the file."""
        data = self._fileobj.read(sizehint)
        if data == "":
            return None
        return self._rfunc(data)
    
    def _write(self,data,flushing=False):
        """Write the given data to the file."""
        self._fileobj.write(self._wfunc(data))

    # Since this is a bytewise translation, the default implementations of
    # _seek(), _tell() and _truncate() will do what we want.




# filelike/__init__.py
#
# Copyright (C) 2006-2009, Ryan Kelly
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
"""

    filelike: a python module for creating and handling file-like objects.

This module takes care of the groundwork for implementing and manipulating
objects that provide a rich file-like interface, including reading, writing,
seeking and iteration.  It also provides a number of useful classes built on
top of this functionality.

The main class is FileLikeBase, which implements the entire file-like interface
on top of primitive _read(), _write(), _seek(), _tell() and _truncate() methods.
Subclasses may implement any or all of these methods to obtain the related
higher-level file behaviors.

It also provides some nifty file-handling functions:

    :open:    mirrors the standard open() function but is much cleverer;
              URLs are automatically fetched, .bz2 files are transparently
              decompressed, and so-on.

    :join:    concatenate multiple file-like objects together so that they
              act like a single file.

    :slice:   access a section of a file-like object as if it were an
              independent file.


The "wrappers" subpackage contains a collection of useful classes built on
top of this framework.  These include:
    
    :Translate:  pass file contents through an arbitrary translation
                 function (e.g. compression, encryption, ...)
                  
    :Decrypt:    on-the-fly reading and writing to an encrypted file
                 (using PEP272 cipher API)

    :UnBZip2:    on-the-fly decompression of bzip'd files
                 (like the standard library's bz2 module, but accepts
                 any file-like object)

As an example of the type of thing this module is designed to achieve, here's
how the Decrypt wrapper can be used to transparently access an encrypted
file::
    
    # Create the decryption key
    from Crypto.Cipher import DES
    cipher = DES.new('abcdefgh',DES.MODE_ECB)
    # Open the encrypted file
    from filelike.wrappers import Decrypt
    f = Decrypt(file("some_encrypted_file.bin","r"),cipher)
    
The object in 'f' now behaves as a file-like object, transparently decrypting
the file on-the-fly as it is read.


The "pipeline" subpackage contains facilities for composing these wrappers
in the form of a unix pipeline.  In the following example, 'f' will read the
first five lines of an encrypted file::
    
    from filelike.pipeline import Decrypt, Head
    f = file("some_encrypted_file.bin") > Decrypt(cipher) | Head(lines=5)


Finally, two utility functions are provided for when code expects to deal with
file-like objects:
    
    :is_filelike(obj):   checks that an object is file-like
    :to_filelike(obj):   wraps a variety of objects in a file-like interface

""" 

__ver_major__ = 0
__ver_minor__ = 4
__ver_patch__ = 1
__ver_sub__ = ""
__ver_tuple__ = (__ver_major__,__ver_minor__,__ver_patch__,__ver_sub__)
__version__ = "%d.%d.%d%s" % __ver_tuple__


from StringIO import StringIO
import urllib2
import urlparse
import tempfile


class NotReadableError(IOError):
    pass
class NotWritableError(IOError):
    pass
class NotSeekableError(IOError):
    pass
class NotTruncatableError(IOError):
    pass


class FileLikeBase(object):
    """Base class for implementing file-like objects.
    
    This class takes a lot of the legwork out of writing file-like objects
    with a rich interface.  It implements the higher-level file-like
    methods on top of five primitive methods: _read, _write, _seek, _tell and
    _truncate. See their docstrings for precise details on how these methods
    behave.
    
    Subclasses then need only implement some subset of these methods for
    rich file-like interface compatability.  They may of course override
    other methods as desired.

    The class is missing the following attributes and methods, which dont
    really make sense for anything but real files:
        
        * fileno()
        * isatty()
        * encoding
        * mode
        * name
        * newlines

    Unlike standard file objects, all read methods share the same buffer
    and so can be freely mixed (e.g. read(), readline(), next(), ...).

    This class understands and will accept the following mode strings,
    with any additional characters being ignored:

        * r    - open the file for reading only.
        * r+   - open the file for reading and writing.
        * r-   - open the file for streamed reading; do not allow seek/tell.
        * w    - open the file for writing only; create the file if
                 it doesn't exist; truncate it to zero length.
        * w+   - open the file for reading and writing; create the file
                 if it doesn't exist; truncate it to zero length.
        * w-   - open the file for streamed writing; do not allow seek/tell.
        * a    - open the file for writing only; create the file if it
                 doesn't exist; place pointer at end of file.
        * a+   - open the file for reading and writing; create the file
                 if it doesn't exist; place pointer at end of file.

    These are mostly standard except for the "-" indicator, which has
    been added for efficiency purposes in cases where seeking can be
    expensive to simulate (e.g. compressed files).  Note that any file
    opened for both reading and writing must also support seeking.
    
    """
    
    def __init__(self,bufsize=1024*64):
        """FileLikeBase Constructor.

        The optional argument 'bufsize' specifies the number of bytes to
        read at a time when looking for a newline character.  Setting this to
        a larger number when lines are long should improve efficiency.
        """
        # File-like attributes
        self.closed = False
        self.softspace = 0
        # Our own attributes
        self._bufsize = bufsize  # buffer size for chunked reading
        self._rbuffer = None     # data that's been read but not returned
        self._wbuffer = None     # data that's been given but not written
        self._sbuffer = None     # data between real & apparent file pos
        self._soffset = 0        # internal offset of file pointer

    def _check_mode(self,mode,mstr=None):
        """Check whether the file may be accessed in the given mode.

        'mode' must be one of "r" or "w", and this function returns False
        if the file-like object has a 'mode' attribute, and it does not
        permit access in that mode.  If there is no 'mode' attribute,
        it defaults to "r+".

        If seek support is not required, use "r-" or "w-" as the mode string.

        To check a mode string other than self.mode, pass it in as the
        second argument.
        """
        if mstr is None:
            try:
                mstr = self.mode
            except AttributeError:
                mstr = "r+"
        if "+" in mstr:
            return True
        if "-" in mstr and "-" not in mode:
            return False    
        if "r" in mode:
            if "r" not in mstr:
                return False    
        if "w" in mode:
            if "w" not in mstr and "a" not in mstr:
                return False
        return True
        
    def _assert_mode(self,mode,mstr=None):
        """Check whether the file may be accessed in the given mode.

        This method is equivalent to _check_assert(), but raises IOError
        instead of returning False.
        """
        if mstr is None:
            try:
                mstr = self.mode
            except AttributeError:
                mstr = "r+"
        if "+" in mstr:
            return True
        if "-" in mstr and "-" not in mode:
            raise NotSeekableError("File does not support seeking.")
        if "r" in mode:
            if "r" not in mstr:
                raise NotReadableError("File not opened for reading")
        if "w" in mode:
            if "w" not in mstr and "a" not in mstr:
                raise NotWritableError("File not opened for writing")
        return True
    
    def flush(self):
        """Flush internal write buffer, if necessary."""
        if self.closed:
            raise IOError("File has been closed")
        if self._check_mode("w-") and self._wbuffer is not None:
            buffered = ""
            if self._sbuffer:
                buffered = buffered + self._sbuffer
                self._sbuffer = None
            buffered = buffered + self._wbuffer
            self._wbuffer = None
            leftover = self._write(buffered,flushing=True)
            if leftover:
                raise IOError("Could not flush write buffer.")
    
    def close(self):
        """Flush write buffers and close the file.

        The file may not be accessed further once it is closed.
        """
        #  Errors in subclass constructors can cause this to be called without
        #  having called FileLikeBase.__init__().  Since we need the attrs it
        #  initialises in cleanup, ensure we call it here.
        if not hasattr(self,"closed"):
            FileLikeBase.__init__(self)
        if not self.closed:
            self.flush()
            self.closed = True

    def __del__(self):
        self.close()

    def __enter__(self):
        return self

    def __exit__(self,exc_type,exc_val,exc_tb):
        self.close()
        return False
    
    def next(self):
        """next() method complying with the iterator protocol.

        File-like objects are their own iterators, with each call to
        next() returning subsequent lines from the file.
        """
        ln = self.readline()
        if ln == "":
            raise StopIteration()
        return ln
    
    def __iter__(self):
        return self

    def truncate(self,size=None):
        """Truncate the file to the given size.

        If <size> is not specified or is None, the current file position is
        used.  Note that this method may fail at runtime if the underlying
        filelike object is not truncatable.
        """
        if "-" in getattr(self,"mode",""):
            raise NotTruncatableError("File is not seekable, can't truncate.")
        if self._wbuffer:
            self.flush()
        if size is None:
            size = self.tell()
        self._truncate(size)

    def seek(self,offset,whence=0):
        """Move the internal file pointer to the given location."""
        if whence > 2 or whence < 0:
            raise ValueError("Invalid value for 'whence': " + str(whence))
        if "-" in getattr(self,"mode",""):
            raise NotSeekableError("File is not seekable.")
        # Ensure that there's nothing left in the write buffer
        if self._wbuffer:
            self.flush()
        # Adjust for any data left in the read buffer
        if whence == 1 and self._rbuffer:
            offset = offset - len(self._rbuffer)
        self._rbuffer = None
        # Adjust for any discrepancy in actual vs apparent seek position
        if whence == 1:
            if self._sbuffer:
                offset = offset + len(self._sbuffer)
            if self._soffset:
                offset = offset + self._soffset
        self._sbuffer = None
        self._soffset = 0
        # Shortcut the special case of staying put
        if offset == 0 and whence == 1:
            return
        # Catch any failed attempts to read while simulating seek
        try:
            # Try to do a whence-wise seek if it is implemented.
            sbuf = None
            try:
                sbuf = self._seek(offset,whence)
            except NotImplementedError:
                # Try to simulate using an absolute seek.
                try:
                    if whence == 1:
                        offset = self._tell() + offset
                    elif whence == 2:
                        if hasattr(self,"size"):
                            offset = self.size + offset
                        else:
                            self._do_read_rest()
                            offset = self.tell() + offset
                    else:
                        # absolute seek already failed, don't try again
                        raise NotImplementedError
                    sbuf = self._seek(offset,0)
                except NotImplementedError:
                    # Simulate by reseting to start
                    self._seek(0,0)
                    self._soffset = offset
            finally:
                self._sbuffer = sbuf
        except NotReadableError:
            raise NotSeekableError("File not readable, can't simulate seek")

    def tell(self):
        """Determine current position of internal file pointer."""
        # Need to adjust for unread/unwritten data in buffers
        pos = self._tell()
        if self._rbuffer:
            pos = pos - len(self._rbuffer)
        if self._wbuffer:
            pos = pos + len(self._wbuffer)
        if self._sbuffer:
            pos = pos + len(self._sbuffer)
        if self._soffset:
            pos = pos + self._soffset
        return pos
    
    def read(self,size=-1):
        """Read at most 'size' bytes from the file.

        Bytes are returned as a string.  If 'size' is negative, zero or
        missing, the remainder of the file is read.  If EOF is encountered
        immediately, the empty string is returned.
        """
        if self.closed:
            raise IOError("File has been closed")
        self._assert_mode("r-")
        return self._do_read(size)

    def _do_read(self,size):
        """Private method to read from the file.

        This method behaves the same as self.read(), but skips some
        permission and sanity checks.  It is intended for use in simulating
        seek(), where we may want to read (and discard) information from
        a file not opened in read mode.

        Note that this may still fail if the file object actually can't
        be read from - it just won't check whether the mode string gives
        permission.
        """
        # If we were previously writing, ensure position is correct
        if self._wbuffer is not None:
            self.seek(0,1)
        # Discard any data that should have been seeked over
        if self._sbuffer:
            s = len(self._sbuffer)
            self._sbuffer = None
            self.read(s)
        elif self._soffset:
            s = self._soffset
            self._soffset = 0
            while s > self._bufsize:
                self._do_read(self._bufsize)
                s -= self._bufsize
            self._do_read(s)
        # Should the entire file be read?
        if size <= 0:
            if self._rbuffer:
                data = [self._rbuffer]
            else:
                data = []
            self._rbuffer = ""
            newData = self._read()
            while newData is not None:
                data.append(newData)
                newData = self._read()
            output = "".join(data)
        # Otherwise, we need to return a specific amount of data
        else:
            if self._rbuffer:
                newData = self._rbuffer
                data = [newData]
            else:
                newData = ""
                data = []
            sizeSoFar = len(newData)
            while sizeSoFar < size:
                newData = self._read(size-sizeSoFar)
                if newData is None:
                    break
                data.append(newData)
                sizeSoFar += len(newData)
            data = "".join(data)
            if sizeSoFar > size:
                # read too many bytes, store in the buffer
                self._rbuffer = data[size:]
                data = data[:size]
            else:
                self._rbuffer = ""
            output = data
        return output

    def _do_read_rest(self):
        """Private method to read the file through to EOF."""
        data = self._do_read(self._bufsize)
        while data != "":
            data = self._do_read(self._bufsize)
        
    def readline(self,size=-1):
        """Read a line from the file, or at most <size> bytes."""
        bits = []
        indx = -1
        sizeSoFar = 0
        while indx == -1:
            nextBit = self.read(self._bufsize)
            bits.append(nextBit)
            sizeSoFar += len(nextBit)
            if nextBit == "":
                break
            if size > 0 and sizeSoFar >= size:
                break
            indx = nextBit.find("\n")
        # If not found, return whole string up to <size> length
        # Any leftovers are pushed onto front of buffer
        if indx == -1:
            data = "".join(bits)
            if size > 0 and sizeSoFar > size:
                extra = data[size:]
                data = data[:size]
                self._rbuffer = extra + self._rbuffer
            return data
        # If found, push leftovers onto front of buffer
        # Add one to preserve the newline in the return value
        indx += 1
        extra = bits[-1][indx:]
        bits[-1] = bits[-1][:indx]
        self._rbuffer = extra + self._rbuffer
        return "".join(bits)
    
    def readlines(self,sizehint=-1):
        """Return a list of all lines in the file."""
        return [ln for ln in self]
    
    def xreadlines(self):
        """Iterator over lines in the file - equivalent to iter(self)."""
        return iter(self)

    def write(self,string):
        """Write the given string to the file."""
        if self.closed:
            raise IOError("File has been closed")
        self._assert_mode("w-")
        # If we were previously reading, ensure position is correct
        if self._rbuffer is not None:
            self.seek(0,1)
        # If we're actually behind the apparent position, we must also
        # write the data in the gap.
        if self._sbuffer:
            string = self._sbuffer + string
            self._sbuffer = None
        elif self._soffset:
            s = self._soffset
            self._soffset = 0
            try:
                string = self._do_read(s) + string
            except NotReadableError:
                raise NotSeekableError("File not readable, could not complete simulation of seek")
            self.seek(0,0)
        if self._wbuffer:
            string = self._wbuffer + string
        leftover = self._write(string)
        if leftover is None:
            self._wbuffer = ""
        else:
            self._wbuffer = leftover
    
    def writelines(self,seq):
        """Write a sequence of lines to the file."""
        for ln in seq:
            self.write(ln)
    
    def _read(self,sizehint=-1):
        """Read approximately <sizehint> bytes from the file-like object.
        
        This method is to be implemented by subclasses that wish to be
        readable.  It should read approximately <sizehint> bytes from the
        file and return them as a string.  If <sizehint> is missing or
        less than or equal to zero, try to read all the remaining contents.
        
        The method need not guarantee any particular number of bytes -
        it may return more bytes than requested, or fewer.  If needed, the
        size hint may be completely ignored.  It may even return an empty
        string if no data is yet available.
        
        Because of this, the method must return None to signify that EOF
        has been reached.  The higher-level methods will never indicate EOF
        until None has been read from _read().  Once EOF is reached, it
        should be safe to call _read() again, immediately returning None.
        """
        raise NotReadableError("Object not readable")
    
    def _write(self,string,flushing=False):
        """Write the given string to the file-like object.
        
        This method must be implemented by subclasses wishing to be writable.
        It must attempt to write as much of the given data as possible to the
        file, but need not guarantee that it is all written.  It may return
        None to indicate that all data was written, or return as a string any
        data that could not be written.
        
        If the keyword argument 'flushing' is true, it indicates that the
        internal write buffers are being flushed, and *all* the given data
        is expected to be written to the file. If unwritten data is returned
        when 'flushing' is true, an IOError will be raised.
        """
        raise NotWritableError("Object not writable")

    def _seek(self,offset,whence):
        """Set the file's internal position pointer, approximately.
 
        This method should set the file's position to approximately 'offset'
        bytes relative to the position specified by 'whence'.  If it is
        not possible to position the pointer exactly at the given offset,
        it should be positioned at a convenient *smaller* offset and the
        file data between the real and apparent position should be returned.

        At minimum, this method must implement the ability to seek to
        the start of the file, i.e. offset=0 and whence=0.  If more
        complex seeks are difficult to implement then it may raise
        NotImplementedError to have them simulated (inefficiently) by
        the higher-level machinery of this class.
        """
        raise NotSeekableError("Object not seekable")

    def _tell(self):
        """Get the location of the file's internal position pointer.

        This method must be implemented by subclasses that wish to be
        seekable, and must return the position of the file's internal
        pointer.

        Due to buffering, the position seen by users of this class
        (the "apparent position") may be different to the position
        returned by this method (the "actual position").
        """
        raise NotSeekableError("Object not seekable")

    def _truncate(self,size):
        """Truncate the file's size to <size>.

        This method must be implemented by subclasses that wish to be
        truncatable.  It must truncate the file to exactly the given size
        or fail with an IOError.

        Note that <size> will never be None; if it was not specified by the
        user then it is calculated as the file's apparent position (which may
        be different to its actual position due to buffering).
        """
        raise NotTruncatableError("Object not truncatable")


class Opener(object):
    """Class allowing clever opening of files.
    
    Instances of this class are callable using inst(filename,mode),
    and are intended as a 'smart' replacement for the standard file
    constructor and open command.  Given a filename and a mode, it returns
    a file-like object representing that file, according to rules such
    as:
        
        * URLs are opened using urllib2
        * files with names ending in ".gz" are gunzipped on the fly
        * etc...
        
    The precise rules that are implemented are determined by two lists
    of functions - openers and decoders.  First, each successive opener
    function is called with the filename and mode until one returns non-None.
    Theese functions must attempt to open the given filename and return it as
    a filelike object.

    Once the file has been opened, it is passed to each successive decoder
    function.  These should return non-None if they perform some decoding
    step on the file.  In this case, they must wrap and return the file-like
    object, modifying its name if appropriate.
    """
    
    def __init__(self,openers=(),decoders=()):
        self.openers = [o for o in openers]
        self.decoders = [d for d in decoders]
    
    def __call__(self,filename,mode="r"):
        # Open the file
        for o in self.openers:
            try:
                f = o(filename,mode)
            except IOError,e:
                f = None
            if f is not None:
                break
        else:
            raise IOError("Could not open file %s in mode '%s'" \
                                                        %(filename,mode))
        # Decode the file as many times as required
        goAgain = True
        while goAgain:
            for d in self.decoders:
                res = d(f)
                if res is not None:
                    f = res
                    break
            else:
                goAgain = False
        # Return the final file object
        return f

##  Create default Opener that uses urllib2.urlopen() and file() as openers
def _urllib_opener(filename,mode):
    if mode not in ("r","r-"):
        return None
    comps = urlparse.urlparse(filename)
    # ensure it's a URL
    if comps[0] == "":
        return None
    f = urllib2.urlopen(filename)
    f.name = f.geturl()
    f.mode = mode
    return f
def _file_opener(filename,mode):
    # Dont open URLS as local files
    comps = urlparse.urlparse(filename)
    if comps[0] and comps[1]:
        return None
    return file(filename,mode)

open = Opener(openers=(_urllib_opener,_file_opener))


def is_filelike(obj,mode="rw"):
    """Test whether an object implements the file-like interface.
    
    'obj' must be the object to be tested, and 'mode' a file access
    mode such as "r", "w" or "rw".  This function returns True if 
    the given object implements the full reading/writing interface
    as required by the given mode, and False otherwise.
    
    If 'mode' is not specified, it deaults to "rw" - that is,
    checking that the full file interface is supported.
    
    This method is not intended for checking basic functionality such as
    existance of read(), but for ensuring the richer interface is
    available.  If only read() or write() is needed, it's probably
    simpler to (a) catch the AttributeError, or (b) use to_filelike(obj)
    to ensure a suitable object.
    """
    # Check reading interface
    if "r" in mode:
        # Special-case for FileLikeBase subclasses
        if isinstance(obj,FileLikeBase):
            if not hasattr(obj,"_read"):
                return False
            if obj._read.im_class is FileLikeBase:
                return False
        else:
            attrs = ("read","readline","readlines","__iter__",)
            for a in attrs:
                if not hasattr(obj,a):
                    return False
    # Check writing interface
    if "w" in mode or "a" in mode:
        # Special-case for FileLikeBase subclasses
        if isinstance(obj,FileLikeBase):
            if not hasattr(obj,"_write"):
                return False
            if obj._write.im_class is FileLikeBase:
                return False
        else:
            attrs = ("write","writelines","close")
            for a in attrs:
                if not hasattr(obj,a):
                    return False
    # Check for seekability
    if "-" not in mode:
        if isinstance(obj,FileLikeBase):
            if not hasattr(obj,"_seek"):
                return False
            if obj._seek.im_class is FileLikeBase:
                return False
        else:
            attrs = ("seek","tell",)
            for a in attrs:
                if not hasattr(obj,a):
                    return False
    return True


class join(FileLikeBase):
    """Class concatenating several file-like objects into a single file.

    This class is similar in spirit to the unix `cat` command, except that
    it produces a file-like object that is readable, writable and seekable
    (so long as the underlying files permit those operations, of course).

    When reading, data is read from each file in turn until it has been
    exhausted.  Seeks and tells are calculated using the individual positions
    of each file.

    When writing, data is spread across each file according to its size,
    and only the last file in the sequence will grow as data is appended.
    This requires that the size of each file can be determined, either by
    checking for a 'size' attribute or using seek/tell.
    """

    def __init__(self,files,mode=None):
        """Filelike join constructor.

        This first argument must be a sequence of file-like objects
        that are to be joined together.  The optional second argument
        specifies the access mode and can be used e.g. to prevent
        writing even when the underlying files are writable.
        """
        super(join,self).__init__()
        if mode:
            self.mode = mode
        self._files = list(files)
        self._curFile = 0
        if mode and "a" in mode:
            self.seek(0,2)

    def close(self):
        super(join,self).close()
        for f in self._files:
            if hasattr(f,"close"):
                f.close()

    def flush(self):
        super(join,self).flush()
        for f in self._files:
            if hasattr(f,"flush"):
                f.flush()

    def _read(self,sizehint=-1):
        data = self._files[self._curFile].read(sizehint)
        if data == "":
            if self._curFile == len(self._files) - 1:
                return None
            else:
                self._curFile += 1
                return self._read(sizehint)
        else:
            return data

    def _write(self,data,flushing=False):
        cf = self._files[self._curFile]
        # If we're at the last file, just write it all out
        if self._curFile == len(self._files) - 1:
            cf.write(data)
            return None
        # Otherwise, we may need to write into multiple files
        pos = cf.tell()
        try:
            size = cf.size
        except AttributeError:
            cf.seek(0,2)
            size = cf.tell()
            cf.seek(pos,0)
        # If the data will all fit in the current file, just write it
        gap = size - pos
        if gap >= len(data):
            cf.write(data)
            return None
        # Otherwise, split up the data and recurse
        cf.write(data[:gap])
        self._curFile += 1
        return self._write(data[gap:],flushing=flushing)

    def _seek(self,offset,whence):
        # Seek-from-end simulated using seek-to-end, then relative seek.
        if whence == 2:
            for f in self._files[self._curFile:]:
                f.seek(0,2)
            self._curFile = len(self._files)-1
            self._seek(offset,1)
        # Absolute seek simulated using tell() and relative seek.
        elif whence == 0:
            offset = offset - self._tell()
            self._seek(offset,1)
        # Relative seek
        elif whence == 1:
            # Working backwards, we simply rewind each file until
            # the offset is small enough to be within the current file
            if offset < 0:
                off1 = self._files[self._curFile].tell()
                while off1 < -1*offset:
                    offset += off1
                    self._files[self._curFile].seek(0,0)
                    # If seeking back past start of first file, stop at zero
                    if self._curFile == 0:
                        return None
                    self._curFile -= 1
                    off1 = self._files[self._curFile].tell()
                self._files[self._curFile].seek(offset,1)
            # Working forwards, we wind each file forward to its end,
            # then seek backwards once we've gone too far.
            elif offset > 0:
                offset += self._files[self._curFile].tell()
                self._files[self._curFile].seek(0,2)
                offset -= self._files[self._curFile].tell()
                while offset > 0:
                    self._curFile += 1
                    self._files[self._curFile].seek(0,2)
                    offset -= self._files[self._curFile].tell()
                self.seek(offset,1)

    def _tell(self):
        return sum([f.tell() for f in self._files[:self._curFile+1]])
 

def slice(f,start=0,stop=None,mode=None,resizable=False):
    """Manipulate a portion of a file-like object.

    This function simply exposes the class filelike.wrappers.Slice
    at the top-level of the module, since it has a nice symmetry
    with the 'join' operation.
    """
    return filelike.wrappers.Slice(f,start,stop,mode,resizable)


def to_filelike(obj,mode="r+"):
    """Convert 'obj' to a file-like object if possible.
    
    This method takes an arbitrary object 'obj', and attempts to
    wrap it in a file-like interface.  This will results in the
    object itself if it is already file-like, or some sort of
    wrapper class otherwise.
    
    'mode', if provided, should specify how the resulting object
    will be accessed.
    
    If the object cannot be converted, ValueError is raised.
    """
    # File-like objects are sutiable on their own
    if is_filelike(obj,mode):
        return obj
    # Strings can be wrapped using StringIO
    if isinstance(obj,basestring):
        return StringIO(obj)
    # Anything with read() and/or write() can be trivially wrapped
    hasRead = hasattr(obj,"read")
    hasWrite = hasattr(obj,"write")
    hasSeek = hasattr(obj,"seek")
    if "r" in mode:
        if "w" in mode or "a" in mode or "+" in mode:
            if hasRead and hasWrite and hasSeek:
                return filelike.wrappers.FileWrapper(obj)
        elif "-" not in mode:
            if hasRead and hasSeek:
                return filelike.wrappers.FileWrapper(obj)
        else:
            if hasRead:
                return filelike.wrappers.FileWrapper(obj)
    if "w" in mode or "a" in mode:
        if "-" not in mode:
            if hasWrite and hasSeek:
                return filelike.wrappers.FileWrapper(obj)
        elif hasWrite:
            return filelike.wrappers.FileWrapper(obj)
    # TODO: lots more could be done here...
    raise ValueError("Could not make object file-like: %s", (obj,))

# Imported here to aoid circular imports
import filelike.wrappers




from filelike.wrappers import PadToBlockSize, UnPadToBlockSize
from filelike import tests, NotSeekableError

from StringIO import StringIO


class Test_PadToBlockSize5(tests.Test_ReadWriteSeek):
    """Testcases for PadToBlockSize with blocksize=5."""

    contents = "this is some sample textZ"
    empty_contents = "ZXXXX"
    text_plain = ["Zhis is sample texty"]
    text_padded = ["Zhis is sample textyZXXXX"]
    blocksize = 5

    def makeFile(self,contents,mode):
        # Careful here - 'contents' should be the contents of the returned
        # file, and is therefore expected to contain the padding.  But for
        # easy testing we allow it to omit the padding and be used directly
        # in the underlying StringIO object.
        idx = contents.rfind("Z")
        if idx < 0:
            idx = len(contents)
        s = StringIO(contents[:idx])
        f = PadToBlockSize(s,self.blocksize,mode=mode)
        def getvalue():
            val = s.getvalue() + "Z"
            if len(val) % self.blocksize != 0:
                val = val + (self.blocksize - (len(val) % self.blocksize))*"X"
            return val
        f.getvalue = getvalue
        return f

    def test_padding(self):
        for (plain,padded) in zip(self.text_plain,self.text_padded):
            f = self.makeFile(padded,"rw")
            self.assert_(len(padded) % self.blocksize == 0)
            self.assertEquals(f._fileobj.getvalue(),plain)

    def test_write_zeds(self):
        f = self.makeFile("","w")
        txt = "test data Z with lots of Z's embedded in it Z"
        f.write("test data Z w")
        f.write("ith lots of Z's e")
        f.write("mbedded in it Z")
        f.write(f._padding(txt))
        f.flush()
        self.assertEquals(f._fileobj.getvalue(),txt)

    def test_write_at_end(self):
        pass


class Test_PadToBlockSize7(Test_PadToBlockSize5):
    """Testcases for PadToBlockSize with blocksize=7."""

    contents = "this is som\n sample textZXXX"
    empty_contents = "ZXXXXXX"
    text_plain = ["Zhis is sample texty"]
    text_padded = ["Zhis is sample textyZ"]
    blocksize = 7


class Test_PadToBlockSize16(Test_PadToBlockSize5):
    """Testcases for PadToBlockSize with blocksize=16."""

    contents = "This is Zome Zample TeZTZXXXXXXX"
    empty_contents = "ZXXXXXXXXXXXXXXX"
    text_plain = ["short"]
    text_padded = ["shortZXXXXXXXXXX"]
    blocksize = 16


class Test_UnPadToBlockSize5(tests.Test_ReadWriteSeek):
    """Testcases for UnPadToBlockSize with blocksize=5."""

    contents = "this is some sample text"
    text_plain = ["Zhis is sample texty"]
    text_padded = ["Zhis is sample textyZXXXX"]
    blocksize = 5

    def makeFile(self,contents,mode):
        f = UnPadToBlockSize(StringIO(""),self.blocksize,mode=mode)
        s = StringIO(contents + f._padding(contents))
        f._fileobj = s
        def getvalue():
            val = s.getvalue()
            idx = val.rfind("Z")
            return val[:idx]
        f.getvalue = getvalue
        return f

    def test_padding(self):
        for (plain,padded) in zip(self.text_plain,self.text_padded):
            f = self.makeFile(plain,"rw")
            self.assertEquals(f._fileobj.getvalue(),padded)

    def test_write_zeds(self):
        f = self.makeFile("","w")
        txt = "test data Z with lots of Z's embedded in it Z"
        f.write("test data Z w")
        f.write("ith lots of Z's e")
        f.write("mbedded in it Z")
        f.flush()
        self.assertEquals(f._fileobj.getvalue(),txt + f._padding(txt))

    def test_read_zeds(self):
        f = self.makeFile("","r")
        txt = "test data Z with lots of Z's embedded in it Z"
        f._fileobj = StringIO(txt + f._padding(txt))
        self.assertEquals(f.read(),txt)


class Test_UnPadToBlockSize7(Test_UnPadToBlockSize5):
    """Testcases for UnPadToBlockSize with blocksize=7."""

    contents = "this is som\n sample text"
    text_plain = ["Zhis is sample texty"]
    text_padded = ["Zhis is sample textyZ"]
    blocksize = 7


class Test_UnPadToBlockSize8(Test_UnPadToBlockSize5):
    """Testcases for UnPadToBlockSize with blocksize=7."""

    contents = "this text is a multiple of eight"
    text_plain = ["Zhis is sample texty"]
    text_padded = ["Zhis is sample textyZXXX"]
    blocksize = 8


class Test_UnPadToBlockSize16(Test_UnPadToBlockSize5):
    """Testcases for UnPadToBlockSize with blocksize=16."""

    contents = "This is Zome Zample TeZTZ"
    text_plain = ["short"]
    text_padded = ["shortZXXXXXXXXXX"]
    blocksize = 16





import unittest
from StringIO import StringIO
import tempfile
import os

import filelike
from filelike import to_filelike, is_filelike, join, wrappers

class ProxyObject:
    def __init__(self,obj):
        self._obj = obj
    def __getattr__(self,attr):
        return getattr(self._obj,attr)


class Test_Read(unittest.TestCase):
    """Generic file-like testcases for readable files."""

    contents = "Once upon a time, in a galaxy far away,\nGuido van Rossum was a space alien."
    empty_contents = ""

    def makeFile(self,contents,mode):
        """This method must create a file of the type to be tested.

        The returned file must have the given contents and mode, and
        have a special method "getvalue" which will return the contents
        of the file as a string.

        By default, a temporary file object is used.  If our tests don't
        pass for the built-in file type, how can we expect to achieve
        anything with them?
        """
        mode = filter(lambda c: c in "rwa+t",mode)
        if "b" not in mode:
            mode = mode + "b"
        if "r" not in mode and "+" not in mode:
            mode = mode + "+"
        (fd,nm) = tempfile.mkstemp()
        os.write(fd,contents)
        os.close(fd)
        f = ProxyObject(open(nm,mode))
        def getvalue():
           pos = f.tell()
           f.seek(0)
           val = f.read()
           f.seek(pos)
           return val
        f.getvalue = getvalue
        return f

    def setUp(self):
        self.file = self.makeFile(self.contents,"r")

    def tearDown(self):
        self.file.close()

    def test_read_all(self):
        c = self.file.read()
        self.assertEquals(c,self.contents)

    def test_read_stream(self):
        f = self.makeFile(self.contents,"r-")
        self.assertEquals(f.read(),self.contents)

    def test_read_size(self):
        c = self.file.read(5)
        self.assertEquals(c,self.contents[:5])
        c = self.file.read(7)
        self.assertEquals(c,self.contents[5:12])

    def test_readline(self):
        c = self.file.readline()
        if self.contents.find("\n") < 0:
            extra = ""
        else:
            extra = "\n"
        self.assertEquals(c,self.contents.split("\n")[0]+extra)

    def test_readlines(self):
        cs = [ln.strip("\n") for ln in self.file.readlines()]
        self.assertEquals(cs,self.contents.split("\n"))

    def test_xreadlines(self):
        cs = [ln.strip("\n") for ln in self.file.xreadlines()]
        self.assertEquals(cs,self.contents.split("\n"))

    def test_read_empty_file(self):
        f = self.makeFile(self.empty_contents,"r")
        self.assertEquals(f.read(),self.empty_contents)

    def test_eof(self):
        self.file.read()
        self.assertEquals(self.file.read(),"")
        self.assertEquals(self.file.read(),"")


class Test_ReadWrite(Test_Read):
    """Generic file-like testcases for writable files."""

    def setUp(self):
        self.file = self.makeFile(self.contents,"r+")

    def test_write(self):
        f = self.makeFile(self.empty_contents,"w")
        f.write(self.contents)
        self.assertEquals(f.tell(),len(self.contents))
        f.flush()
        self.assertEquals(f.getvalue(),self.contents)
        f.close()

    def test_append(self):
        f = self.makeFile(self.empty_contents,"a")
        f.write(self.contents)
        self.assertEquals(f.tell(),len(self.contents))
        f.flush()
        self.assertEquals(f.getvalue(),self.contents)
        f.close()

    def test_write_stream(self):
        f = self.makeFile(self.empty_contents,"w-")
        f.write(self.contents)
        self.assertEquals(f.tell(),len(self.contents))
        f.flush()
        self.assertEquals(f.getvalue(),self.contents)
        f.close()

    def test_write_read(self):
        self.file.write("hello")
        self.file.seek(0,1)
        c = self.file.read()
        self.assertEquals(c,self.contents[5:])

    def test_read_write_read(self):
        c = self.file.read(5)
        self.assertEquals(c,self.contents[:5])
        self.file.write("hello")
        c = self.file.read(5)
        self.assertEquals(c,self.contents[10:15])


class Test_ReadWriteSeek(Test_ReadWrite):
    """Generic file-like testcases for seekable files."""

    def test_seek_tell(self):
        self.assertEquals(self.file.tell(),0)
        self.file.seek(7)
        self.assertEquals(self.file.tell(),7)
        self.assertEquals(self.file.read(),self.contents[7:])
        self.file.seek(0,0)
        self.assertEquals(self.file.tell(),0)

    def test_read_write_seek(self):
        c = self.file.read(5)
        self.assertEquals(c,self.contents[:5])
        self.file.seek(0,1)
        self.file.write("hello")
        self.assertEquals(self.file.tell(),10)
        self.file.seek(0)
        self.assertEquals(self.file.tell(),0)
        c = self.file.read(10)
        self.assertEquals(c,self.contents[:5] + "hello")

    def test_seek_cur(self):
        self.assertEquals(self.file.tell(),0)
        self.file.seek(7,1)
        self.assertEquals(self.file.tell(),7)
        self.file.seek(7,1)
        self.assertEquals(self.file.tell(),14)
        self.file.seek(-5,1)
        self.assertEquals(self.file.tell(),9)

    def test_seek_end(self):
        self.assertEquals(self.file.tell(),0)
        self.file.seek(-7,2)
        self.assertEquals(self.file.tell(),len(self.contents)-7)
        self.file.seek(3,1)
        self.assertEquals(self.file.tell(),len(self.contents)-4)

    def test_write_at_end(self):
        self.assertEquals(self.file.tell(),0)
        self.file.seek(0,2)
        self.file.write("testable")
        self.file.seek(0,0)
        self.assertEquals(self.file.read(),self.contents+"testable")

    def test_write_twice(self):
        f = self.makeFile(self.empty_contents,"w")
        f.write(self.contents)
        self.assertEquals(f.tell(),len(self.contents))
        f.flush()
        self.assertEquals(f.getvalue(),self.contents)
        f.seek(-5,2)
        self.assertEquals(f.tell(),len(self.contents) - 5)
        f.write(self.contents[-5:])
        f.flush()
        self.assertEquals(f.getvalue(),self.contents)


class Test_StringIO(Test_ReadWriteSeek):
    """Run our testcases against StringIO, basically to test the tests."""

    def makeFile(self,contents,mode):
        f = StringIO(contents)
        f.seek(0)
        def xreadlines():
            for ln in f.readlines():
                yield ln
        f.xreadlines = xreadlines
        return f


class Test_Join(Test_ReadWriteSeek):
    """Run our testcases against filelike.join."""

    def makeFile(self,contents,mode):
        files = []
        files.append(StringIO(contents[0:5]))
        files.append(StringIO(contents[5:8]))
        files.append(StringIO(contents[8:]))
        f = join(files)
        def getvalue():
            return "".join([s.getvalue() for s in files])
        f.getvalue = getvalue
        return f


class Test_IsTo(unittest.TestCase):
    """Tests for is_filelike/to_filelike."""

    def test_isfilelike(self):
        """Test behaviour of is_filelike."""
        self.assert_(is_filelike(tempfile.TemporaryFile()))
        self.assert_(is_filelike(tempfile.TemporaryFile("r"),"r"))
        self.assert_(is_filelike(tempfile.TemporaryFile("r"),"w"))
        self.assert_(is_filelike(StringIO()))

    def test_tofilelike_read(self):
        """Test behavior of to_filelike for mode "r-"."""
        class F:
            def read(self,sz=-1):
                return ""
        f = to_filelike(F(),"r-")
        self.assertEquals(f.__class__,wrappers.FileWrapper)
        self.assertEquals(f.read(),"")
        self.assertRaises(ValueError,to_filelike,F(),"r")
        self.assertRaises(ValueError,to_filelike,F(),"w-")
        self.assertRaises(ValueError,to_filelike,F(),"rw")

    def test_tofilelike_readseek(self):
        """Test behavior of to_filelike for mode "r"."""
        class F:
            def read(self,sz=-1):
                return ""
            def seek(self,offset,whence):
                pass
        f = to_filelike(F(),"r")
        self.assertEquals(f.__class__,wrappers.FileWrapper)
        self.assertEquals(f.read(),"")
        self.assertRaises(ValueError,to_filelike,F(),"w")
        self.assertRaises(ValueError,to_filelike,F(),"w-")
        self.assertRaises(ValueError,to_filelike,F(),"rw")

    def test_tofilelike_write(self):
        """Test behavior of to_filelike for mode "w-"."""
        class F:
            def write(self,data):
                pass
        f = to_filelike(F(),"w-")
        self.assertEquals(f.__class__,wrappers.FileWrapper)
        self.assertRaises(ValueError,to_filelike,F(),"w")
        self.assertRaises(ValueError,to_filelike,F(),"r")
        self.assertRaises(ValueError,to_filelike,F(),"r-")
        self.assertRaises(ValueError,to_filelike,F(),"rw")

    def test_tofilelike_writeseek(self):
        """Test behavior of to_filelike for mode "w"."""
        class F:
            def write(self,data):
                pass
            def seek(self,offset,whence):
                pass
        f = to_filelike(F(),"w")
        self.assertEquals(f.__class__,wrappers.FileWrapper)
        self.assertRaises(ValueError,to_filelike,F(),"r")
        self.assertRaises(ValueError,to_filelike,F(),"r-")

    def test_tofilelike_readwrite(self):
        """Test behavior of to_filelike for mode "rw"."""
        class F:
            def write(self,data):
                pass
            def read(self,sz=-1):
                return ""
            def seek(self,offset,whence):
                pass
        f = to_filelike(F(),"rw")
        self.assertEquals(f.__class__,wrappers.FileWrapper)
        self.assertEquals(f.read(),"")

    def test_tofilelike_stringio(self):
        """Test behaviour of to_filelike on StringIO instances."""
        f = to_filelike(StringIO())
        self.assert_(isinstance(f,StringIO))

    def test_tofilelike_string(self):
        """Test behaviour of to_filelike on strings."""
        f = to_filelike("testing")
        self.assert_(isinstance(f,StringIO))
        self.assertEquals(f.read(),"testing")
        

class Test_Docs(unittest.TestCase):
    """Unittests for our documentation."""

    def test_README(self):
        """Ensure that the README is in sync with the docstring.

        This test should always pass; if the README is out of sync it just
        updates it with the contents of filelike.__doc__.
        """
        dirname = os.path.dirname
        readme = os.path.join(dirname(dirname(__file__)),"README.txt")
        if not os.path.isfile(readme):
            f = open(readme,"wb")
            f.write(filelike.__doc__.encode())
            f.close()
        else:
            f = open(readme,"rb")
            if f.read() != filelike.__doc__:
                f.close()
                f = open(readme,"wb")
                f.write(filelike.__doc__.encode())
                f.close()



def build_test_suite():
    suite = unittest.TestSuite()
    for (dirnm,_,files) in os.walk(os.path.dirname(filelike.__file__)):
        for fname in files:
            if fname.startswith("test") and fname.endswith(".py"):
                testname = fname[:-3]
                modnames = []
                remainder = dirnm
                while "filelike" not in  modnames:
                    (remainder,modparent) = os.path.split(remainder)
                    modnames.insert(0,modparent)
                testmod = __import__(".".join(modnames),fromlist=(testname,))
                testmod = getattr(testmod,testname)
                for nm in dir(testmod):
                    if nm.startswith("Test"):
                        suite.addTest(unittest.makeSuite(getattr(testmod,nm)))
    return suite



# filelike/wrappers/fixedblocksize.py
#
# Copyright (C) 2006-2008, Ryan Kelly
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
"""

    filelike.wrappers.fixedblocksize:  read/write only on block boundaries
    
This module provides the filelike wrapper 'FixedBlockSize' to ensure that
reads/writes to the underlying file are only performed at block boundaries.

""" 

import filelike
from filelike.wrappers import FileWrapper


class FixedBlockSize(FileWrapper):
    """Class reading/writing to files at a fixed block size.
    
    This file wrapper can be used to read or write to a file-like
    object at a specific block size.  All reads request strings
    whose length is a multiple of the block size, and all writes
    pass on strings of a similar nature.  This could be useful, for
    example, to write data to a cipher function without manually 
    chunking text to match the cipher's block size.
    
    No padding is added to the file if its length is not a multiple 
    of the blocksize.  This might cause things to fail when this file
    is flushed or closed, since an incorrectly-sized string could be
    given in this case.
    """
    
    def __init__(self,fileobj,blocksize,mode=None):
        self.blocksize = blocksize
        super(FixedBlockSize,self).__init__(fileobj,mode)
    
    def _round_up(self,num):
        """Round <num> up to a multiple of the block size."""
        if num % self.blocksize == 0:
            return num
        return ((num/self.blocksize)+1) * self.blocksize
    
    def _round_down(self,num):
        """Round <num> down to a multiple of the block size."""
        if num % self.blocksize == 0:
            return num
        return (num/self.blocksize) * self.blocksize

    def _read(self,sizehint=-1):
        """Read approximately <sizehint> bytes from the file."""
        if sizehint >= 0:
            sizehint = self._round_up(sizehint)
        data = self._fileobj.read(sizehint)
        if data == "":
            return None
        return data

    def _write(self,data,flushing=False):
        """Write the given string to the file.

        When flushing data to the file, it may need to be padded to the
        block size.  We attempt to read additional data from the
        underlying file to use for the padding.
        """
        size = self._round_down(len(data))
        self._fileobj.write(data[:size])
        if len(data) == size:
            return ""
        if not flushing:
            return data[size:]
        # Flushing, so we need to try to pad the data with existing contents.
        # If we can't find such contents, just write at non-blocksize.
        if self._check_mode("r"):
            nextBlock = self._fileobj.read(self.blocksize)
            self._fileobj.seek(-1*len(nextBlock),1)
        else:
            nextBlock = ""
        padstart = len(data) - size
        self._fileobj.write(data[size:] + nextBlock[padstart:])
        # Seek back to start of previous block, if the file is readable.
        if self._check_mode("r"):
            self.seek(padstart - self.blocksize,1)
        return ""

    # TODO: primitive implementation of relative seek
    def _seek(self,offset,whence):
        """Absolute seek, repecting block boundaries.

        This method performs an absolute file seek to the block boundary
        closest to (but not exceeding) the specified offset.
        """
        if whence != 0:
            raise NotImplementedError
        boundary = self._round_down(offset)
        self._fileobj.seek(boundary,0)
        if boundary == offset:
            return ""
        else:
            data = self._fileobj.read(self.blocksize)
            diff = offset - boundary - len(data)
            if diff > 0:
                # Seeked past end of file.  Actually do this on fileobj, so
                # that it will raise an error if appropriate.
                self._fileobj.seek(diff,1)
                self._fileobj.seek(-1*diff,1)
            self._fileobj.seek(-1*len(data),1)
            return data[:(offset-boundary)]
 




# filelike/wrappers/unix.py
#
# Copyright (C) 2006-2008, Ryan Kelly
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
"""

    filelike.wrappers.unix:  wrapper classes emulating unix commands
    
This module provides filelike wrapper classes inspired by standard unix
command-line programs.  Currently these include:

    * Head:    read/write only a the first N bytes or lines in a file

""" 

import filelike
from filelike.wrappers import FileWrapper


class Head(FileWrapper):
    """Wrapper acting like unix "head" command.
    
    This wrapper limits the amount of data returned from or written to the
    underlying file based on the number of bytes and/or lines.  This class
    currently does not support seeking or simultaneous read/write.
    
    NOTE: no guarantees are made about the amount of data read *from*
          the underlying file, only about the amount of data returned to
          the calling function.
    """
    
    def __init__(self,fileobj,mode=None,bytes=None,lines=None):
        """Head wrapper constructor.

        The arguments 'bytes' and 'lines' specify the maximum number
        of bytes and lines to be read or written.  Reading/writing
        will terminate when one of the given values has been exceeded.
        Any extraneous data is simply discarded.
        """
        super(Head,self).__init__(fileobj,mode)
        self._maxBytes = bytes
        self._maxLines = lines
        self._bytesR = 0
        self._linesR = 0
        self._bytesW = 0
        self._linesW = 0
        self._finishedR = False
        self._finishedW = False
    
    def _read(self,sizehint=-1):
        if self._finishedR:
            return None
        if sizehint <= 0 or sizehint > self._bufsize:
            sizehint = self._bufsize
        data = self._fileobj.read(sizehint)
        if data == "":
            self._finishedR = True
            return data
        nBytes = len(data)
        newBytes = self._bytesR + nBytes
        if self._maxBytes is not None and newBytes >= self._maxBytes:
            data = data[:self._maxBytes - self._bytesR]
            self._finishedR = True
        nLines = data.count("\n")
        newLines = self._linesR + nLines
        if self._maxLines is not None and newLines >= self._maxLines:
            limit = self._maxLines - self._linesR
            lines = data.split("\n")
            if len(lines) > limit:
                data = "\n".join(lines[:limit]) + "\n"
            else:
                data = "\n".join(lines[:limit])
            self._finishedR = True
        self._bytesR = newBytes
        self._linesR = newLines
        return data

    def _write(self,data,flushing=True):
        if self._finishedW:
            return None
        nBytes = len(data)
        nLines = data.count("\n")
        newBytes = self._bytesW + nBytes
        newLines = self._linesW + nLines
        if self._maxBytes is not None and newBytes >= self._maxBytes:
            data = data[:self._maxBytes - self._bytesW]
            self._finishedW = True
        elif self._maxLines is not None and newLines >= self._maxLines:
            limit = self._maxLines - self._linesW
            lines = data.split("\n")
            if len(lines) > limit:
                data = "\n".join(lines[:limit]) + "\n"
            else:
                data = "\n".join(lines[:limit])
            self._finishedW = True
        self._bytesW = newBytes
        self._linesW = newLines
        self._fileobj.write(data)
        return None





 
import unittest
from filelike.wrappers import FixedBlockSize
from filelike import tests

from StringIO import StringIO

class Test_FixedBlockSize5(tests.Test_ReadWriteSeek):
    """Testcases for the FixedBlockSize class, with blocksize 5."""

    blocksize = 5
    
    def makeFile(self,contents,mode):
        f = StringIO(contents)
        f.seek(0)
        class BSFile:
            """Simulate reads/writes, asserting correct blocksize."""
            def read(s,size=-1):
                self.assert_(size < 0 or size % self.blocksize == 0)
                return f.read(size)
            def write(s,data):
                if not s._flushing:
                    self.assert_(len(data)%self.blocksize == 0)
                f.write(data)
            def seek(s,offset,whence):
                f.seek(offset,whence)
            def tell(s):
                return f.tell()
            def flush(self):
                f.flush()
        bsf = BSFile()
        bsf._flushing = False
        fbsf = FixedBlockSize(bsf,self.blocksize)
        # Patch it to indicate when it's flushing, so we don't raise errors
        oldflush = fbsf.flush
        def newflush():
            bsf._flushing = True
            oldflush()
            bsf._flushing = False
        fbsf.flush = newflush
        def getvalue():
            return f.getvalue()
        fbsf.getvalue = getvalue
        return fbsf


class Test_FixedBlockSize7(Test_FixedBlockSize5):
    """Testcases for the FixedBlockSize class, with blocksize 7."""
    blocksize = 7


class Test_FixedBlockSize24(Test_FixedBlockSize5):
    """Testcases for the FixedBlockSize class, with blocksize 24."""
    blocksize = 24





from filelike.wrappers import Translate, BytewiseTranslate
from filelike import tests, NotSeekableError

import unittest
from StringIO import StringIO


class Test_Translate(tests.Test_ReadWriteSeek):
    """Testcases for the Translate class, with null translation func."""
    
    def makeFile(self,contents,mode):
        def noop(string):
            return string
        s = StringIO(contents)
        f = Translate(s,noop,mode=mode)
        def getvalue():
            return s.getvalue()
        f.getvalue = getvalue
        return f


class Test_BytewiseTranslate(tests.Test_ReadWriteSeek):
    """Testcases for the BytewiseTranslate class."""
    
    def makeFile(self,contents,mode):
        def rot13(string):
            return string.encode("rot13")
        s = StringIO(contents.encode("rot13"))
        f = BytewiseTranslate(s,rot13,mode=mode)
        def getvalue():
            return s.getvalue().encode("rot13")
        f.getvalue = getvalue
        return f





import unittest
from StringIO import StringIO

from filelike.pipeline import *

class Test_Pipeline(unittest.TestCase):
    """Testcases for the construction of pipelines."""
    
    def setUp(self):
        from Crypto.Cipher import DES
        self.cipher = DES.new('abcdefgh',DES.MODE_ECB)
        self.plaintext = "Guido van Rossum is a space alien." + "\0"*6
        self.ciphertext = "\x11,\xe3Nq\x8cDY\xdfT\xe2pA\xfa\xad\xc9s\x88\xf3,\xc0j\xd8\xa8\xca\xe7\xe2I\xd15w\x1d\xfe\x92\xd7\xca\xc9\xb5r\xec"
        self.plainfile = StringIO(self.plaintext)
        self.cryptfile = StringIO(self.ciphertext)
        self.outfile = StringIO()

    def tearDown(self):
        pass

    def test_ReaderLine(self):
        """Test a simple reading pipeline."""
        pf = self.ciphertext > Decrypt(self.cipher) | Head(bytes=10)
        txt = pf.read()
        self.assertEquals(txt,self.plaintext[:10])

    def test_WriterLine(self):
        """Test a simple writer pipeline."""
        pf = Decrypt(self.cipher) | Head(bytes=15) | FixedBlockSize(10) > self.outfile
        pf.write(self.plaintext)
        pf.flush()
        txt = self.outfile.getvalue()
        self.assertEquals(txt,self.ciphertext[:15])




# filelike/wrappers/__init__.py
#
# Copyright (C) 2006-2009, Ryan Kelly
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
"""

    filelike.wrappers:  wrapper classes modifying file-like objects
    
This module builds on the basic functionality of the filelike module to
provide a collection of useful classes.  These include:
    
    * Translate:  pass file contents through an arbitrary translation
                  function (e.g. compression, encryption, ...)
                  
    * Decrypt:    on-the-fly reading and writing to an encrypted file
                  (using PEP272 cipher API)

    * UnBZip2:    on-the-fly decompression of bzip'd files
                  (like the standard library's bz2 module, but accepts
                  any file-like object)
 
As an example of the type of thing this module is designed to achieve, here's
how to use the Decrypt wrapper to transparently access an encrypted file:
    
    # Create the decryption key
    from Crypto.Cipher import DES
    cipher = DES.new('abcdefgh',DES.MODE_ECB)
    # Open the encrypted file
    f = Decrypt(file("some_encrypted_file.bin","r"),cipher)
    
The object in 'f' now behaves as a file-like object, transparently decrypting
the file on-the-fly as it is read.

""" 

import filelike
from filelike import FileLikeBase


class FileWrapper(FileLikeBase):
    """Base class for objects that wrap a file-like object.
    
    This class provides basic functionality for implementing file-like
    objects that wrap another file-like object to alter its functionality
    in some way.  It takes care of house-keeping duties such as flushing
    and closing the wrapped file.

    Access to the wrapped file is given by the private member _fileobj.
    By convention, the subclass's constructor should accept this as its
    first argument and pass it to its superclass's constructor in the
    same position.
    
    This class provides a basic implementation of _read() and _write()
    which just calls read() and write() on the wrapped object.  Subclasses
    will probably want to override these.
    """

    _append_requires_overwrite = False

    def __init__(self,fileobj,mode=None):
        """FileWrapper constructor.
        
        'fileobj' must be a file-like object, which is to be wrapped
        in another file-like object to provide additional functionality.
        
        If given, 'mode' must be the access mode string under which
        the wrapped file is to be accessed.  If not given or None, it
        is looked up on the wrapped file if possible.  Otherwise, it
        is not set on the object.
        """
        # This is used for working around flush/close inefficiencies
        self._closing = False
        super(FileWrapper,self).__init__()
        self._fileobj = fileobj
        if mode is None:
            self.mode = getattr(fileobj,"mode","r+")
        else:
            self.mode = mode
        self._validate_mode()
        # Copy useful attributes of the fileobj
        if hasattr(fileobj,"name"):
            self.name = fileobj.name
        # Respect append-mode setting
        if "a" in self.mode:
            if self._check_mode("r"):
                self._fileobj.seek(0)
            self.seek(0,2)

    def _validate_mode(self):
        """Check that various file-mode conditions are satisfied."""
        #  If append mode requires overwriting the underlying file,
        #  if must not be opened in append mode.
        if self._append_requires_overwrite:
            if self._check_mode("w"):
                if "a" in getattr(self._fileobj,"mode",""):
                    raise ValueError("Underlying file can't be in append mode")

    def __del__(self):
        #  Errors in subclass constructors could result in this being called
        #  without invoking FileWrapper.__init__.  Establish some simple
        #  invariants to prevent errors in this case.
        if not hasattr(self,"_fileobj"):
            self._fileobj = None
        if not hasattr(self,"_closing"):
            self._closing = False
        #  Close the wrapper and the underlying file independently, so the
        #  latter is still closed on cleanup even if the former errors out.
        try:
            super(FileWrapper,self).close()
        except Exception:
            if hasattr(getattr(self,"_fileobj",None),"close"):
                self._fileobj.close()
            raise
        
    def close(self):
        """Close the object for reading/writing."""
        #  The superclass implementation of this will call flush(),
        #  which calls flush() on our wrapped object.  But we then call
        #  close() on it, which will call its flush() again!  To avoid
        #  this inefficiency, our flush() will not flush the wrapped
        #  fileobj when we're closing.
        self._closing = True
        super(FileWrapper,self).close()
        if hasattr(self._fileobj,"close"):
            self._fileobj.close()

    def flush(self):
        """Flush the write buffers of the file."""
        super(FileWrapper,self).flush()
        if not self._closing and hasattr(self._fileobj,"flush"):
            self._fileobj.flush()
    
    def _read(self,sizehint=-1):
        data = self._fileobj.read(sizehint)
        if data == "":
            return None
        return data

    def _write(self,string,flushing=False):
        return self._fileobj.write(string)

    def _seek(self,offset,whence):
        self._fileobj.seek(offset,whence)

    def _tell(self):
        return self._fileobj.tell()

    def _truncate(self,size):
        return self._fileobj.truncate(size)

##  Import the various classes from our sub-modules.

from filelike.wrappers.debug import Debug

from filelike.wrappers.translate import Translate, BytewiseTranslate

from filelike.wrappers.fixedblocksize import FixedBlockSize

from filelike.wrappers.padtoblocksize import PadToBlockSize, UnPadToBlockSize

from filelike.wrappers.crypto import Encrypt, Decrypt

from filelike.wrappers.buffer import Buffer, FlushableBuffer

from filelike.wrappers.compress import BZip2, UnBZip2

from filelike.wrappers.unix import Head

from filelike.wrappers.slice import Slice





from filelike.wrappers import Buffer, FlushableBuffer
from filelike import tests

import unittest
from StringIO import StringIO


def get_buffered_value(f):
    if f._check_mode("r"):
        f._read_rest()
    pos = f._buffer.tell()
    f._buffer.seek(0)
    val = f._buffer.read()
    f._buffer.seek(pos)
    return val


def def_getvalue_maybe_buffered(f,s,trans_s=None,trans_b=None):
    if isinstance(f._fileobj,Buffer):
        if trans_b:
            def getvalue():
                return trans_b(get_buffered_value(f._fileobj))
        else:
            def getvalue():
                return get_buffered_value(f._fileobj)
    else:
        if trans_s:
            def getvalue():
                return trans_s(s.getvalue())
        else:
            def getvalue():
                return s.getvalue()
    return getvalue


class Test_Buffer(tests.Test_ReadWriteSeek):
    """Testcases for the Buffer class."""
    
    def makeFile(self,contents,mode):
        s = StringIO(contents)
        if "a" in mode:
            s.seek(0,2)
        f = Buffer(s,mode)
        def getvalue():
            return get_buffered_value(f)
        f.getvalue = getvalue
        return f

    def test_buffer_w(self):
        f = self.makeFile("","w")
        s = f._fileobj
        close = s.close
        def noop():
            pass
        s.close = noop
        f.write("testing")
        f.flush()
        self.assertEquals(f.getvalue(),"testing")
        self.assertEquals(s.getvalue(),"")
        f.close()
        self.assertEquals(s.getvalue(),"testing")

    def test_buffer_rw(self):
        f = self.makeFile("testing","r+")
        s = f._fileobj
        close = s.close
        def noop():
            pass
        s.close = noop
        f.write("hello")
        f.flush()
        self.assertEquals(f.getvalue(),"hellong")
        self.assertEquals(s.getvalue(),"testing")
        f.close()
        self.assertEquals(s.getvalue(),"hellong")

    def test_buffer_a(self):
        f = self.makeFile("hello","a")
        s = f._fileobj
        close = s.close
        def noop():
            pass
        s.close = noop
        f.write("testing")
        f.flush()
        self.assertEquals(f.getvalue(),"testing")
        self.assertEquals(s.getvalue(),"hello")
        f.close()
        self.assertEquals(s.getvalue(),"hellotesting")

    def test_buffer_ra(self):
        f = self.makeFile("hello","a+")
        s = f._fileobj
        close = s.close
        def noop():
            pass
        s.close = noop
        f.write("testing")
        f.flush()
        self.assertEquals(f.getvalue(),"hellotesting")
        self.assertEquals(s.getvalue(),"hello")
        f.close()
        self.assertEquals(s.getvalue(),"hellotesting")


class Test_Buffer_rollover(Test_Buffer):
    """Testcases for the Buffer class with rollover to tempfile."""
    
    def makeFile(self,contents,mode):
        s = StringIO(contents)
        if "a" in mode:
            s.seek(0,2)
        f = Buffer(s,mode,max_size_in_memory=1)
        def getvalue():
            return get_buffered_value(f)
        f.getvalue = getvalue
        return f


class Test_FlushableBuffer(tests.Test_ReadWriteSeek):
    """Testcases for the FlushableBuffer class."""
    
    def makeFile(self,contents,mode):
        s = StringIO(contents)
        if "a" in mode:
            s.seek(0,2)
        f = FlushableBuffer(s,mode)
        def getvalue():
            return s.getvalue()
        f.getvalue = getvalue
        return f

    def test_buffer_w(self):
        f = self.makeFile("","w")
        s = f._fileobj
        close = s.close
        def noop():
            pass
        s.close = noop
        f.write("testing")
        f.close()
        self.assertEquals(s.getvalue(),"testing")

    def test_buffer_rw(self):
        f = self.makeFile("testing","r+")
        s = f._fileobj
        close = s.close
        def noop():
            pass
        s.close = noop
        f.write("hello")
        f.close()
        self.assertEquals(s.getvalue(),"hellong")

    def test_buffer_a(self):
        f = self.makeFile("hello","a")
        s = f._fileobj
        close = s.close
        def noop():
            pass
        s.close = noop
        f.write("testing")
        f.close()
        self.assertEquals(s.getvalue(),"hellotesting")

    def test_buffer_ra(self):
        f = self.makeFile("hello","a+")
        self.assertEquals(f._fileobj.tell(),5)
        self.assertEquals(f.tell(),5)
        s = f._fileobj
        close = s.close
        def noop():
            pass
        s.close = noop
        f.write("testing")
        f.close()
        self.assertEquals(s.getvalue(),"hellotesting")


class Test_FlushableBuffer_rollover(Test_FlushableBuffer):
    """Testcases for the FlushableBuffer class with rollover to tempfile."""
    
    def makeFile(self,contents,mode):
        s = StringIO(contents)
        if "a" in mode:
            s.seek(0,2)
        f = FlushableBuffer(s,mode,max_size_in_memory=1)
        def getvalue():
            return get_buffered_value(f)
        f.getvalue = getvalue
        return f




# filelike/wrappers/padtoblocksize.py
#
# Copyright (C) 2006-2008, Ryan Kelly
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
"""

    filelike.wrappers.padtoblocksize:  pad a file to a given blocksize
    
This module provides the dual filelike wrappers 'PadToBlockSize' and 
'UnPadToBlockSize' to handle padding of a file to a specified blocksize.

""" 

import filelike
from filelike.wrappers import FileWrapper


class PadToBlockSize(FileWrapper):
    """Class padding files to a fixed block size.
    
    This file wrapper can be used to pad a file to a specific block size.
    The file data is followed by a 'Z', then as many 'X' bytes as needed
    to meet the block size.  This is automatically added when reading,
    and stripped when writing.  The dual of this class is UnPadToBlockSize.

    This class does not necessarily align reads or writes along block
    boundaries - use the FixedBlockSize wrapper to achieve this.
    """

    def __init__(self,fileobj,blocksize,mode=None):
        self.blocksize = blocksize
        self._pad_read = ""
        self._pad_unread = ""
        super(PadToBlockSize,self).__init__(fileobj,mode)
        if "a" in self.mode:
            # Position at the start of the padding, since that's
            # where any additional writes need to happen
            self._pad_unread = self._pad_read
            self._pad_read = ""
            self._sbuffer = None
    
    def _round_up(self,num):
        """Round <num> up to a multiple of the block size."""
        if num % self.blocksize == 0:
            return num
        return ((num/self.blocksize)+1) * self.blocksize
    
    def _round_down(self,num):
        """Round <num> down to a multiple of the block size."""
        if num % self.blocksize == 0:
            return num
        return (num/self.blocksize) * self.blocksize
    
    def _padding(self,data):
        """Get the padding needed to make 'data' match the blocksize."""
        padding = "Z"
        size = self._round_up(len(data)+1)
        padding = padding + ("X"*(size-len(data)-1))
        return padding

    def _read(self,sizehint=-1):
        # If there is unread padding, return that
        if self._pad_unread:
            data = self._pad_unread
            self._pad_read = self._pad_read + data
            self._pad_unread = ""
            return data
        # If the padding has been read, return EOF
        if self._pad_read:
            return None
        # Always read at the blocksize, as it makes padding easier
        if sizehint > 0:
            sizehint = self._round_up(sizehint)
        data = self._fileobj.read(sizehint)
        if sizehint < 0 or len(data) < sizehint:
            self._pad_unread = self._padding(data)
        return data

    def _write(self,string,flushing=False):
        # Check whether this could contain the padding block, which
        # will need to be removed.
        zIdx = string.rfind("Z")
        maybePad = zIdx >= len(string) - self.blocksize - 1
        for c in string[zIdx+1:]:
            if c != "X":
                maybePad = False
                break
        # If it may contain the padding block, don't write those blocks
        # just yet.  Otherwise, write as much as possible.
        if maybePad:
            size = self._round_down(zIdx)
        else:
            size = self._round_down(len(string))
        self._fileobj.write(string[:size])
        leftover = string[size:]
        # If there's no leftover, well, that was easy :-)
        if not leftover:
            return None
        # If we're not flushing, we can delay writing the leftovers.
        if not flushing:
            return leftover
        # If we are flushing, we need to write the leftovers.
        # If we're in the middle of the file, write out a complete block
        # using the existing file contents.  Only works if readable...
        if self._check_mode("r"):
            lenNB = self._round_up(len(leftover))
            nextBlock = self._fileobj.read(lenNB)
            self._fileobj.seek(-1*len(nextBlock),1)
            if lenNB == len(nextBlock):
                padstart = len(leftover)
                self._fileobj.write(leftover + nextBlock[padstart:])
                self.seek(padstart - lenNB,1)
                return None
        # Otherwise, we must be at the end of the file.
        # Remove the padding data from the leftovers
        if maybePad:
            zIdx = leftover.rfind("Z")
            data = leftover[:zIdx]
            padding = self._padding(data)
            padstop = len(leftover) - zIdx
            self._fileobj.write(data)
            self._pad_read = padding[:padstop]
            self._pad_unread = padding[padstop:]
        return None

    def _seek(self,offset,whence):
        """Seek to approximately 'offset' bytes from start of file.

        This method implements absolute seeks and will not seek to
        positions beyond the end of the file.  If you try to seek past
        the file and its padding, you'll be placed at EOF.
        """
        if whence > 0:
            # TODO: implementing these shouldn't be that hard...
            raise NotImplementedError
        self._fileobj.seek(0,0)
        self._pad_unread = ""
        self._pad_read = ""
        if offset == 0:
            return None
        # Slow simulation of seek by actually re-reading all the data.
        boundary = self._round_down(offset)
        bytes_read = 0
        while bytes_read < boundary:
            data = self._fileobj.read(min(self._bufsize,boundary-bytes_read))
            if data == "":
                break
            bytes_read += len(data)
        # If the boundary is not within the file, we must have seeked right
        # to (or past) the end of the padding.  So just position at end.
        if bytes_read < boundary:
            data = ""
            self._pad_read = self._padding("A"*(bytes_read % self.blocksize))
            return None
        # Otherwise, we may have to return some data from the underlying file
        pos = self._fileobj.tell()
        data = self._fileobj.read(offset-boundary)
        self._fileobj.seek(-1*len(data),1)
        assert self._fileobj.tell() == pos, "peeking failed"
        diff = offset - (len(data) + bytes_read)
        assert diff >= 0, "peeking failed"
        if diff > 0:
            # The target offset is somewhere in the padding
            padding = self._padding(data)
            return data + padding[:diff]
        return data

    def _tell(self):
        return self._fileobj.tell() + len(self._pad_read)

    def _truncate(self,size):
        if size % self.blocksize != 0:
            msg = "PadToBlockSize must be truncated to a multiple of " \
                  "the blocksize"
            raise IOError(msg)
        pos = self._fileobj.tell()
        if size <= pos:
            self._pad_read = self._pad_unread = None
        else:
            self._fileobj.seek(0,2)
            fsize = self._fileobj.tell()
            self._fileobj.seek(pos,0)
            if size > fsize:
                msg = "PadToBlockSize can't truncate past end of file"
                raise IOError(msg)
        self._fileobj.truncate(size)


class UnPadToBlockSize(FileWrapper):
    """Class removing block-size padding from a file.
    
    This file wrapper can be used to reverse the effects of PadToBlockSize,
    removing extraneous padding data when reading, and adding it back in
    when writing.
    """

    _append_requires_overwrite = True
    
    def __init__(self,fileobj,blocksize,mode=None):
        self.blocksize = blocksize
        self._pad_seen = ""
        super(UnPadToBlockSize,self).__init__(fileobj,mode)

    def _round_up(self,num):
        """Round <num> up to a multiple of the block size."""
        if num % self.blocksize == 0:
            return num
        return ((num/self.blocksize)+1) * self.blocksize
    
    def _round_down(self,num):
        """Round <num> down to a multiple of the block size."""
        if num % self.blocksize == 0:
            return num
        return (num/self.blocksize) * self.blocksize
    
    def _padding(self,data):
        """Get the padding needed to make 'data' match the blocksize."""
        padding = "Z"
        size = self._round_up(len(data)+1)
        padding = padding + ("X"*(size-len(data)-1))
        return padding
    
    def _read(self,sizehint=-1):
        """Read approximately <sizehint> bytes from the file."""
        if sizehint >= 0:
            sizehint = self._round_up(sizehint)
        data = self._fileobj.read(sizehint)
        # If we might be near the end, read far enough ahead to find the pad
        zIdx = data.rfind("Z")
        if sizehint >= 0:
            if data == "X"*self.blocksize:
                newData = self._fileobj.read(self.blocksize)
                sizehint += self.blocksize
                data = data + newData
            if zIdx >= 0:
                while zIdx >= (len(data) - self.blocksize - 1):
                    newData = self._fileobj.read(self.blocksize)
                    sizehint += self.blocksize
                    data = data + newData
                    zIdx = data.rfind("Z")
                    if newData == "":
                        break
        # Return the data, stripping the pad if we're at the end
        if data == "":
            return None
        if sizehint < 0 or len(data) < sizehint:
            if zIdx < 0:
                assert len(data) <= self.blocksize
                return None
            else:
                self._pad_seen = data[zIdx:]
                return data[:zIdx]
        else:
            return data

    def _write(self,data,flushing=False):
        """Write the given string to the file."""
        #  To ensure the pad is written, we always return non-empty leftovers.
        #  This forces a flushing write on file close.
        if data == "":
            return ""
        size = self._round_down(len(data)-1)
        self._fileobj.write(data[:size])
        leftover = data[size:]
        if not flushing:
            return leftover
        # Flushing, so we need to pad the data.  If the file is readable,
        # check to see if we're in the middle and pad using existing data.
        if self._check_mode("r"):
            lenNB = self._round_up(len(leftover)+1)
            nextBlock = self._fileobj.read(lenNB)
            self._fileobj.seek(-1*len(nextBlock),1)
            if lenNB == len(nextBlock):
                padstart = len(leftover)
                self._fileobj.write(leftover + nextBlock[padstart:])
                self.seek(padstart - lenNB,1)
                return None
        # Otherwise, we must be at the end of the file.
        padding = self._padding(leftover)
        self._fileobj.write(leftover + padding)
        self._pad_seen = padding
        return None

    def _seek(self,offset,whence):
        if whence > 0:
            raise NotImplementedError
        self._fileobj.seek(0,0)
        self._pad_seen = ""
        if offset == 0:
            return None
        data = self._fileobj.read(offset)
        eof = data.rfind("Z")
        if len(data) < offset:
            offset = eof
        elif eof != -1 and eof > len(data) - self.blocksize - 1:
            extra = self._fileobj.read(self.blocksize+1)
            data = data + extra
            if len(extra) <= self.blocksize:
                eof = data.rfind("Z")
                if eof < offset:
                    offset = eof
        boundary = self._round_down(offset)
        self._fileobj.seek(boundary-len(data),1)
        return data[boundary:offset]

    def _tell(self):
        return self._fileobj.tell() - len(self._pad_seen)

    def _truncate(self,size):
        msg = "UnPadToBlockSize objects are not truncatable"
        raise filelike.NotTruncatableError(msg)




# filelike/wrappers/buffer.py
#
# Copyright (C) 2009, Ryan Kelly
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
"""

    filelike.wrappers.buffer:  buffering of streams to create a file
    
This module provides the filelike wrapper 'Buffer', which routes reads
and writes through a separate buffer file.  This allows the full file-like
interface to be provided, including seek() and tell(), while guaranteeing
that the underlying file is treated like a stream, with only read() and
write() being called.

The subclass 'FlushableBuffer' additionally assumes that the underlying
stream can be reset back to position zero, allowing flushes to be performed.

""" 

import os
import sys

import filelike
from filelike.wrappers import FileWrapper

try:
    from tempfile import SpooledTemporaryFile
except ImportError:
    from tempfile import TemporaryFile
    def SpooledTemporaryFile(max_size=None,*args,**kwds):
        return TemporaryFile(*args,**kwds)


class Buffer(FileWrapper):
    """Class implementing buffering of input and output streams.
    
    This class uses a separate buffer file to hold the contents of the
    underlying file while they are being manipulated.  As data is read
    it is duplicated into the buffer, and data is written from the buffer
    back to the file on close.
    """
    
    def __init__(self,fileobj,mode=None,max_size_in_memory=1024*8):
        """Buffered file wrapper constructor."""
        self._buffer = SpooledTemporaryFile(max_size=max_size_in_memory)
        self._in_eof = False
        self._in_pos = 0
        self._was_truncated = False
        super(Buffer,self).__init__(fileobj,mode)

    def _buffer_size(self):
        try:
            return len(self._buffer.file.getvalue())
        except AttributeError:
            return os.fstat(self._buffer.fileno()).st_size

    def _buffer_chunks(self):
        chunk = self._buffer.read(16*1024)
        if chunk == "":
            yield chunk
        else:
            while chunk != "":
                yield chunk
                chunk = self._buffer.read(16*1024)

    def _write_out_buffer(self):
        if self._check_mode("r"):
            self._read_rest()
            if "a" in self.mode:
                self._buffer.seek(self._in_pos)
                self._fileobj.seek(self._in_pos)
            else:
                self._fileobj.seek(0)
                self._buffer.seek(0)
        else:
            self._buffer.seek(0)
        if self._was_truncated:
            self._fileobj.truncate(0)
            self._was_truncated = False
        for chunk in self._buffer_chunks():
            self._fileobj.write(chunk)
 
    def flush(self):
        # flush the buffer; we only write to the underlying file on close
        self._buffer.flush()

    def close(self):
        if self.closed:
            return
        if self._check_mode("w"):
            self._write_out_buffer()
        super(Buffer,self).close()
        self._buffer.close()

    def _read(self,sizehint=-1):
        #  First return any data available from the buffer.
        #  Since we don't flush the buffer after every write, certain OSes
        #  (guess which!) will happily read junk data from the end of it.
        #  Instead, we explicitly read only up to self._in_pos.
        if not self._in_eof:
            buffered_size = self._in_pos - self._buffer.tell()
            if sizehint >= 0:
                buffered_size = min(sizehint,buffered_size)
        else:
            buffered_size = sizehint
        data = self._buffer.read(buffered_size)
        if data != "":
            return data
        # Then look for more data in the underlying file
        if self._in_eof:
            return None
        data = self._fileobj.read(sizehint)
        self._in_pos += len(data)
        self._buffer.write(data)
        if sizehint < 0 or len(data) < sizehint:
            self._in_eof = True
            self._buffer.flush()
        return data

    def _write(self,data,flushing=False):
        self._buffer.write(data)
        if self._check_mode("r") and not self._in_eof:
            diff = self._buffer.tell() - self._in_pos
            if diff > 0:
                junk = self._fileobj.read(diff)
                self._in_pos += len(junk)
                if len(junk) < diff:
                    self._in_eof = True
                    self._buffer.flush()
    
    def _seek(self,offset,whence):
        # Ensure we've read enough to simply do the seek on the buffer
        if self._check_mode("r") and not self._in_eof:
            if whence == 0:
                if offset > self._in_pos:
                    self._read_rest()
            if whence == 1:
                if self._buffer.tell() + offset > self._in_pos:
                    self._read_rest()
            if whence == 2:
                self._read_rest()
        # Then just do it on the buffer...
        self._buffer.seek(offset,whence)

    def _tell(self):
        return self._buffer.tell()

    def _truncate(self,size):
        if self._check_mode("r") and not self._in_eof:
            if size > self._in_pos:
                self._read_rest()
        self._in_eof = True
        try:
            self._buffer.truncate(size)
        except TypeError:
            et,ev,tb = sys.exc_info()
            # SpooledTemporaryFile.truncate() doesn't accept size paramter.
            try:
                self._buffer._file.truncate(size)
            except Exception:
                raise et,ev,tb
        # StringIO objects don't truncate to larger size correctly.
        if hasattr(self._buffer,"_file"):
            _file = self._buffer._file
            if hasattr(_file,"getvalue"):
                if len(_file.getvalue()) != size:
                    curpos = _file.tell()
                    _file.seek(0,2)
                    _file.write("\x00" * (size - len(_file.getvalue())))
                    _file.seek(curpos)
        self._was_truncated = True

    def _read_rest(self):
        """Read the rest of the input stream."""
        if self._in_eof:
            return
        pos = self._buffer.tell()
        self._buffer.seek(0,2)
        data = self._fileobj.read(self._bufsize)
        while data:
            self._in_pos += len(data)
            self._buffer.write(data)
            data = self._fileobj.read(self._bufsize)
        self._in_eof = True 
        self._buffer.flush()
        self._buffer.seek(pos)


class FlushableBuffer(Buffer):
    """Buffered file wrapper that supports flusing.

    This subclass of Buffer assumes that the underlying file object can
    be reset to position 0, allowing calls to flush() to write out to
    the underlying file.
    """

    _append_requires_overwite = True

    def __init__(self,fileobj,mode=None,max_size_in_memory=1024*8):
        super(FlushableBuffer,self).__init__(fileobj,mode,max_size_in_memory)
        if "a" in self.mode and not self._check_mode("r"):
            self._start_pos = self._fileobj.tell()

    def flush(self):
        if self._check_mode("w-"):
            pos = self._buffer.tell()
            self._write_out_buffer()
            self._buffer.seek(pos)
        self._buffer.flush()
        # Skip Buffer.flush, as it doesn't call parent class methods
        super(Buffer,self).flush()

    def close(self):
        if self.closed:
            return
        # Don't call Buffer.close, it will call _write_out_buffer, but
        # that's done by the implicit flush() in this case.
        super(Buffer,self).close()

    def _write_out_buffer(self):
        if self._check_mode("r"):
            self._read_rest()
            if "a" in self.mode:
                self._buffer.seek(self._in_pos)
                self._fileobj.seek(self._in_pos)
            else:
                self._fileobj.seek(0)
                self._buffer.seek(0)
        else:
            if "a" in self.mode:
                self._fileobj.seek(self._start_pos)
            else:
                self._fileobj.seek(0)
            self._buffer.seek(0)
        if self._was_truncated:
            self._fileobj.truncate(0)
            self._was_truncated = False
        for chunk in self._buffer_chunks():
            self._fileobj.write(chunk)






from filelike.wrappers import Slice
from filelike import tests

import unittest
from StringIO import StringIO
 

class Test_Slice_Whole(tests.Test_ReadWriteSeek):
    """Testcases for the Slice wrapper class."""

    def makeFile(self,contents,mode,start=0,stop=None,resizable=False):
        s = StringIO(contents)
        f = Slice(s,start,stop,resizable=resizable,mode=mode)
        def getvalue():
            val = s.getvalue()
            if stop:
                val = val[:f.stop]
            if start:
                val = val[f.start:]
            return val
        f.getvalue = getvalue
        return f


class Test_Slice_Start(Test_Slice_Whole):
    """Testcases for the Slice wrapper class with a start offset."""

    def makeFile(self,contents,mode):
        c2 = "testing" + contents
        return super(Test_Slice_Start,self).makeFile(c2,mode,7)


class Test_Slice_StartStop(Test_Slice_Whole):
    """Testcases for the Slice wrapper class with both start and stop."""

    def makeFile(self,contents,mode):
        c2 = "testing" + contents + "hello"
        return super(Test_Slice_StartStop,self).makeFile(c2,mode,7,-5)

    def test_write(self):
        method = super(Test_Slice_StartStop,self).test_write
        self.assertRaises(IOError,method)

    def test_append(self):
        method = super(Test_Slice_StartStop,self).test_write
        self.assertRaises(IOError,method)

    def test_write_stream(self):
        method = super(Test_Slice_StartStop,self).test_write_stream
        self.assertRaises(IOError,method)

    def test_write_at_end(self):
        method = super(Test_Slice_StartStop,self).test_write_at_end
        self.assertRaises(IOError,method)

    def test_write_twice(self):
        method = super(Test_Slice_StartStop,self).test_write_twice
        self.assertRaises(IOError,method)


class Test_Slice_StartStopResize(Test_Slice_Whole):
    """Testcases for the Slice wraper, with resizable stop."""

    contents = "a simple test string"

    def makeFile(self,contents,mode):
        c2 = "testing" + contents + "hello"
        return super(Test_Slice_StartStopResize,self).makeFile(c2,mode,7,-5,True)
    
    def test_resizability(self):
        """Test that resizing slices works correctly."""
        #  By default, can't write beyond end of slice.
        f = Slice(StringIO("mytestdata"),start=2,stop=6)
        f.write("TE")
        f.seek(0)
        self.assertEquals(f.read(),"TEst")
        self.assertEquals(f._fileobj.getvalue(),"myTEstdata")
        f.seek(0)
        self.assertRaises(IOError,f.write,"TESTDATA")
        self.assertEquals(f._fileobj.getvalue(),"myTESTdata")
        # Resizability allows data to be overwritten
        f = Slice(StringIO("mytestdata"),start=2,stop=6,resizable=True)
        f.write("TESTDA")
        self.assertEquals(f._fileobj.getvalue(),"myTESTDAta")
        self.assertEquals(f.stop,8)
        



# filelike/pipeline/__init__.py
#
# Copyright (C) 2006-2008, Ryan Kelly
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
"""

    filelike.pipeline:  manipulate file-like objects in unix pipeline style
    
This module utilises python's operator overloading magic to allow file-like
wrappers such as those found in filelike.wrappers to be composed in the style
of a unix pipeline.

Ideas based on the following ASPN Cookbook Recipie:
    
    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/276960

The base class PipelineEntry implements the operator magic, and a mirror
class is provided for each wrapper in filelike.wrappers. To allow additional
wrappers to be used, use pipeline() on the class:
    
    class MyNewWrapper(FileWrapper):
        ...
    
    MyNewWrapper = pipeline(MyNewWrapper)
    

As an example, here's how to print the first 20 lines of an encrypted file:
    
    for ln in file("enc_file.bin","r") > Decrypt(key) | Head(lines=20)
        print ln

Some folks would say this aids readability when using long combinations of
wrappers.  A lot would probably call it a horrible abuse of operator semantics.
Mostly I consider it a fun hack.

The following pipline operators are supported:
    
    >     Read or write from a wrapper into a file-like object
    >>    Write from a wrapper into a file-like object in append mode
    |     Read/write from one wrapper into another

Because of the way python's operator lookup works, > is used for both read
and write.  At the beginning of a pipeline, it indicates that data is to
be read from the file.  At the end, it indicates data should be written
to the file.

"""

import filelike

class PipelineEntry:
    """Class implementing a step in a file-like pipeline.
    
    Objects of this class may form a stage in a file-like pipeline.  Typically
    this will be either the first entry in the pipeline, or an entry following
    a readable file-like object.
    
    Each instance of this class must implement the method _create(fileobj),
    which creates and returns an appropriate wrapper around the given
    file-like object.
    """
    
    def __init__(self,cls,*args,**kwds):
        """PipelineEntry constructor.

        'cls' is the class of the file-like wrapper to create.  Additional
        args may be specified and will be passed in after the requisite
        'fileobj' first argument in the class's constructor.
        """
        self._cls = cls
        self._args = args
        self._kwds = kwds
    
    def _create(self,fileobj,mode=None):
        """Create instance of the FileWrapper over given file object."""
        if mode is not None and not self._kwds.has_key("mode"):
            kwds = self._kwds.copy()
            kwds["mode"] = mode
        else:
            kwds = self._kwds
        return self._cls(fileobj,*self._args,**kwds)
    
    def __lt__(self,obj):
        """Implement read-from-file pipeline stage.
        
        'obj' must not be a PipelineEntry, and must be coercable
        to a file-like object using filelike.to_filelike() with
        mode "r".
        
        A new file-like object will be returned that reads from the
        opened file-like object.
        """
        if isinstance(obj,PipelineEntry):
            raise ValueError("Cannot read from existing pipeline entry.")
        obj = filelike.to_filelike(obj,"r")
        return self._create(obj,"r")
    
    def __gt__(self,obj):
        """Implement write-to-file pipeline stage.
        
        'obj' must not be a PipelineEntry, and must be coercable
        to a file-like object using filelike.to_filelike() with
        mode "w".
        
        A new file-like wrapper will be returned that writes to
        the given object.
        """
        if isinstance(obj,PipelineEntry):
            raise ValueError("Cannot write to existing pipeline entry.")
        obj = filelike.to_filelike(obj,"w")
        return self._create(obj,"w")

    def __rshift__(self,obj):
        """Implement append-to-file pipeline stage.
        
        This behaves as __gt__, but opens the file in append mode.
        """
        if isinstance(obj,PipelineEntry):
            raise ValueError("Cannot write to existing pipeline entry.")
        obj = filelike.to_filelike(obj,"a")
        return self._create(obj,"a")

    def __or__(self,obj):
        """Implement left-hand pipe segment in pipeling stage.
        
        'obj' should be the next pipeline stage.  This starts a
        PipelineStack.
        """
        return PipelineStack(self,obj)

class PipelineStack:
    """Class collecting pipeline entries to be read/written to.
    
    This class collects a stack of pipeline entries. These will
    be converted to a filelike object in the appropriate order
    when the write/read-to-file pipeline stage is encountered.
    """
    
    def __init__(self,first,second):
        """PipelineStack constructor.

        'first' and 'second' must be the initial two stages in the
        pipeline.
        """
        self._stages = [second,first]
    
    def __or__(self,obj):
        """Implement left-hand pipe segment in pipeling stage.
        
        'obj' should be the next pipeline stage, which is added to the stack.
        """
        self._stages.append(obj)
        return self
    
    def __gt__(self,obj):
        """Implement write-to-file pipeline stage.
        
        'obj' must not be a PipelineEntry, and must be coercable
        to a file-like object using filelike.to_filelike() with
        mode "w".
        
        A new file-like wrapper will be returned that writes to
        the given object.
        """
        if isinstance(obj,PipelineEntry):
            raise ValueError("Cannot write to existing pipeline entry.")
        obj = filelike.to_filelike(obj,"w")
        while len(self._stages) > 0:
            next = self._stages.pop(0)
            obj = next._create(obj,"w")
        return obj

    def __lt__(self,obj):
        """Implement read-from-file pipeline stage.
        
        'obj' must not be a PipelineEntry, and must be coercable
        to a file-like object using filelike.to_filelike() with
        mode "w".
        
        A new file-like wrapper will be returned that writes to
        the given object.
        """
        if isinstance(obj,PipelineEntry):
            raise ValueError("Cannot read from existing pipeline entry.")
        obj = filelike.to_filelike(obj,"r")
        while len(self._stages) > 0:
            next = self._stages.pop()
            obj = next._create(obj,"r")
        return obj

    def __rshift__(self,obj):
        """Implement append-to-file pipeline stage.
        
        This behaves as __gt__, but opens the file in append mode.
        """
        if isinstance(obj,PipelineEntry):
            raise ValueError("Cannot write to existing pipeline entry.")
        obj = filelike.to_filelike(obj,"a")
        while len(self._stages) > 0:
            next = self._stages.pop(0)
            obj = next._create(obj,"a")
        return obj


def pipeline(cls):
    """Create a PipelineEntry factory function using given class."""
    def create_entry(*args,**kwds):
        return PipelineEntry(cls,*args,**kwds)
    return create_entry


##  Create a PipelineEntry factory for each wrapper
##  defined in filelike.wrappers
from filelike import wrappers
for nm in dir(wrappers):
    cls = getattr(wrappers,nm)
    try:
        if issubclass(cls,wrappers.FileWrapper):
            if cls is not wrappers.FileWrapper:
                globals()[nm] = pipeline(cls)
    except TypeError:
        pass




# filelike/wrappers/compress.py
#
# Copyright (C) 2006-2009, Ryan Kelly
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
"""

    filelike.wrappers.compress:  wrapper classes handling compressed files
    
This module provides the filelike wrappers 'BZip2' and 'UnBZip2' for dealing
with files compressed in bz2 format.  It also provides some base classes for
building other compression wrappers.

""" 

import filelike
from filelike.wrappers import FileWrapper
from filelike.wrappers.translate import Translate
from filelike.wrappers.buffer import FlushableBuffer

import bz2

class Decompress(FileWrapper):
    """Abstract base class for decompressing files.

    Instances of this class represent the decompressed version of a file - 
    all data read from the file is decompressed on demand, and all data
    written to the file is compressed.

    Subclases must provide compress() and decompress() methods.
    """

    def __init__(self,fileobj,mode=None):
        if mode is None:
            try:
                mode = fileobj.mode
            except AtributeError:
                mode = "r+"
        myFileObj = None
        if "r" in mode:
            if "w" not in mode and "a" not in mode and "+" not in mode:
                # Nice and easy, just a streaming decompress on read
                myFileObj = Translate(fileobj,mode=mode,rfunc=self.decompress)
        else:
            if "-" in mode:
                # Nice and easy, just a streaming compress on write
                myFileObj = Translate(fileobj,mode=mode,wfunc=self.compress)
        if not myFileObj:
            # Rats, writing + seekabilty == inefficient.
            # Operating in a buffer is the only sensible option
            myFileObj = Translate(fileobj,mode=mode,rfunc=self.decompress,wfunc=self.compress)
            myFileObj = FlushableBuffer(myFileObj,mode=mode)
        super(Decompress,self).__init__(myFileObj,mode=mode)

    def _read(self,sizehint=-1):
        data = super(Decompress,self)._read(sizehint)
        return data


class Compress(FileWrapper):
    """Abstract base class for compressing files.

    Instances of this class represent the compressed version of a file - 
    all data read from the file is compressed on demand, and all data
    written to the file is decompressed.

    Subclases must provide compress() and decompress() methods.
    """

    def __init__(self,fileobj,mode=None):
        if mode is None:
            try:
                mode = fileobj.mode
            except AtributeError:
                mode = "r+"
        myFileObj = None
        if "r" in mode:
            if "w" not in mode and "a" not in mode and "+" not in mode:
                # Nice and easy, just a streaming compress on read
                myFileObj = Translate(fileobj,mode=mode,rfunc=self.compress)
        else:
            if "-" in mode:
                # Nice and easy, just a streaming decompress on write
                myFileObj = Translate(fileobj,mode=mode,wfunc=self.decompress)
        if not myFileObj:
            # Rats, writing + seekabilty == inefficient
            # Operating in a buffer is the only sensible option
            myFileObj = Translate(fileobj,mode=mode,rfunc=self.compress,wfunc=self.decompress)
            myFileObj = FlushableBuffer(myFileObj,mode=mode)
        super(Compress,self).__init__(myFileObj,mode=mode)


class BZip2Mixin(object):
    """Mixin for Compress/Decompress subclasses using Bzip2."""

    def __init__(self,*args,**kwds):
        if not hasattr(self,"compresslevel"):
            self.compresslevel = 9
        # Compression function with flush and reset.
        c = [bz2.BZ2Compressor()]
        def compress(data):
            if data == "":
                return ""
            return c[0].compress(data)
        def c_flush():
            return c[0].flush()
        def c_reset():
            c[0] = bz2.BZ2Compressor()
        compress.flush = c_flush
        compress.reset = c_reset
        self.compress = compress
        # Decompression funtion with reset
        d = [bz2.BZ2Decompressor()]
        def decompress(data):
            if data == "":
                return ""
            return d[0].decompress(data)
        def d_reset():
            d[0] = bz2.BZ2Decompressor()
        decompress.reset = d_reset
        self.decompress = decompress
        # These can now be used by superclass constructors
        super(BZip2Mixin,self).__init__(*args,**kwds)


class UnBZip2(BZip2Mixin,Decompress):
    """Class for reading and writing to a un-bziped file.
        
    This class behaves almost exactly like the bz2.BZ2File class from
    the standard library, except that it accepts an arbitrary file-like
    object.  All reads from the file are decompressed, all writes are
    compressed.
    """
    
    def __init__(self,fileobj,mode=None,compresslevel=9):
        self.compresslevel = compresslevel
        super(UnBZip2,self).__init__(fileobj,mode=mode)


class BZip2(BZip2Mixin,Compress):
    """Class for reading and writing a bziped file.
        
    This class is the dual of UnBZip2 - it compresses read data, and
    decompresses written data.  Thus BZip2(f) is the compressed version
    of f.
    """
    
    def __init__(self,fileobj,mode=None,compresslevel=9):
        self.compresslevel = compresslevel
        super(BZip2,self).__init__(fileobj,mode=mode)


##  Add handling of .bz2 files to filelike.open()
def _BZip2_decoder(fileobj):
    """Decoder function for handling .bz2 files with filelike.open"""
    if not fileobj.name.endswith(".bz2"):
        return None
    f = UnBZip2(fileobj)
    f.name = fileobj.name[:-4]
    return f
filelike.open.decoders.append(_BZip2_decoder)


class NullZipMixin(object):
    """Mixin for Compress/Decompress subclasses using NullZip."""

    def __init__(self,*args,**kwds):
        class NullCompressor(object):
            def compress(self,data):
                return data
            def decompress(self,data):
                return data
            def flush(self):
                return ""
        # Compression function with flush and reset.
        c = [NullCompressor()]
        def compress(data):
            if data == "":
                return ""
            return c[0].compress(data)
        def c_flush():
            return c[0].flush()
        def c_reset():
            c[0] = NullCompressor()
        compress.flush = c_flush
        compress.reset = c_reset
        self.compress = compress
        # Decompression funtion with reset
        d = [NullCompressor()]
        def decompress(data):
            if data == "":
                return ""
            return d[0].decompress(data)
        def d_reset():
            d[0] = NullCompressor()
        decompress.reset = d_reset
        self.decompress = decompress
        # These can now be used by superclass constructors
        super(NullZipMixin,self).__init__(*args,**kwds)


class UnNullZip(NullZipMixin,Decompress):
    """Class for reading and writing to a null-ziped file."""
    pass


class NullZip(NullZipMixin,Compress):
    """Class for reading and writing a null-ziped file."""
    pass





import sys
import distribute_setup
distribute_setup.use_setuptools()
from setuptools import setup, find_packages


setup_kwds = {}
setup_kwds["test_suite"] = "filelike.tests.build_test_suite"
if sys.version_info > (3,):
    setup_kwds["use_2to3"] = True


info = {}
try:
    next = next
except NameError:
    def next(i):
        return i.next()
src = open("filelike/__init__.py")
lines = []
ln = next(src)
while "__version__" not in ln:
    lines.append(ln)
    ln = next(src)
while "__version__" in ln:
    lines.append(ln)
    ln = next(src)
exec("".join(lines),info)


NAME = "filelike"
VERSION = info["__version__"]
DESCRIPTION = "Classes for creating and wrapping file-like objects"
AUTHOR = "Ryan Kelly"
AUTHOR_EMAIL = "ryan@rfk.id.au"
URL = "http://www.rfk.id.au/software/filelike/"
LICENSE = "LGPL"
KEYWORDS = "file filelike file-like filter crypt compress"
LONG_DESC = info["__doc__"]


PACKAGES = find_packages()
EXT_MODULES = []
PKG_DATA = {}


setup(name=NAME,
      version=VERSION,
      author=AUTHOR,
      author_email=AUTHOR_EMAIL,
      url=URL,
      description=DESCRIPTION,
      long_description=LONG_DESC,
      keywords=KEYWORDS,
      packages=PACKAGES,
      ext_modules=EXT_MODULES,
      package_data=PKG_DATA,
      license=LICENSE,
      **setup_kwds
     )




#
# Copyright (C) 2006-2008, Ryan Kelly
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
"""

    filelike.wrappers.crypto:  wrapper classes for cryptography
    
This module provides the filelike wrappers 'Encrypt' and 'Decrypt' for
dealing with encrypted files.

""" 

import filelike
from filelike.wrappers import FileWrapper
from filelike.wrappers.translate import Translate, BytewiseTranslate
from filelike.wrappers.buffer import FlushableBuffer
from filelike.wrappers.fixedblocksize import FixedBlockSize


class Decrypt(FileWrapper):
    """Class for reading and writing to an encrypted file.
    
    This class accesses an encrypted file using a ciphering object
    compliant with PEP272: "API for Block Encryption Algorithms".
    All reads from the file are automatically decrypted, while writes
    to the file and automatically encrypted.  Thus, Decrypt(fobj)
    can be seen as the decrypted version of the file-like object fobj.
    
    Because this class is implemented on top of FixedBlockSize, it
    assumes that the file size is a multiple of the block size.  If this
    is not appropriate, wrap it with UnPadToBlockSize.
    
    There is a dual class, Encrypt, where all reads are encrypted
    and all writes are decrypted.  This would be used, for example, to
    encrypt the contents of an existing file using a series of read()
    operations.
    """

    def __init__(self,fileobj,cipher,mode=None):
        """Decrypt Constructor.

        'fileobj' is the file object with encrypted contents, and 'cipher'
        is the cipher object to be used.  Other arguments are passed through
        to FileWrapper.__init__
        """
        if mode is None:
            try:
                mode = fileobj.mode
            except AttributeError:
                mode = "r+"
        self._cipher = cipher
        if cipher.mode == 1:
            # MODE_ECB is a bytewise translation
            myFileObj = BytewiseTranslate(fileobj,mode=mode,
                                                  rfunc=cipher.decrypt,
                                                  wfunc=cipher.encrypt)
            myFileObj = FixedBlockSize(myFileObj,cipher.block_size,mode=mode)
            if self._check_mode("w",mode) and "-" not in mode:
                if not self._check_mode("r",mode):
                    myFileObj = FlushableBuffer(myFileObj,mode=mode)
        else:
            # Other modes are stateful translations.
            # To reset them, we simply reset the initialisation vector
            initialIV = cipher.IV
            def rfunc(data):
                return cipher.decrypt(data)
            def wfunc(data):
                return cipher.encrypt(data)
            def reset():
                cipher.IV = initialIV
            rfunc.reset = reset
            wfunc.reset = reset
            myFileObj = Translate(fileobj,mode=mode,rfunc=rfunc,wfunc=wfunc)
            myFileObj = FixedBlockSize(myFileObj,cipher.block_size,mode=mode)
            #  To allow writes with seeks, we need to buffer.
            #  TODO: find a way around this.
            if self._check_mode("rw",mode):
                myFileObj = FlushableBuffer(myFileObj,mode=mode)
            elif self._check_mode("w",mode) and "-" not in mode:
                myFileObj = FlushableBuffer(myFileObj,mode=mode)
        super(Decrypt,self).__init__(myFileObj,mode=mode)


class Encrypt(FileWrapper):
    """Class for reading and writing to an decrypted file.
    
    This class accesses a decrypted file using a ciphering object
    compliant with PEP272: "API for Block Encryption Algorithms".
    All reads from the file are automatically encrypted, while writes
    to the file are automatically decrypted.  Thus, Encrypt(fobj)
    can be seen as the encrypted version of the file-like object fobj.

    Because this class is implemented on top of FixedBlockSize, it
    assumes that the file size is a multiple of the block size.  If this
    is not appropriate, wrap the underlying file object with PadToBlockSize.
    You will need to write the padding data yourself.
    
    There is a dual class, Decrypt, where all reads are decrypted
    and all writes are encrypted.  This would be used, for example, to
    decrypt the contents of an existing file using a series of read()
    operations.
    """

    def __init__(self,fileobj,cipher,mode=None):
        """Encrypt Constructor.

        'fileobj' is the file object with decrypted contents, and 'cipher'
        is the cipher object to be used.  Other arguments are passed through
        to FileWrapper.__init__
        """
        self._cipher = cipher
        if cipher.mode == 1:
            # MODE_ECB is a bytewise translation
            myFileObj = BytewiseTranslate(fileobj,mode=mode,
                                                  rfunc=cipher.encrypt,
                                                  wfunc=cipher.decrypt)
            myFileObj = FixedBlockSize(myFileObj,cipher.block_size,mode=mode)
            if self._check_mode("w",mode) and "-" not in mode:
                if not self._check_mode("r",mode):
                    myFileObj = FlushableBuffer(myFileObj,mode=mode)
        else:
            # Other modes are stateful translations.
            # To reset them, we simply reset the initialisation vector
            initialIV = cipher.IV
            def rfunc(data):
                return cipher.encrypt(data)
            def wfunc(data):
                return cipher.decrypt(data)
            def reset():
                cipher.IV = initialIV
            rfunc.reset = reset
            wfunc.reset = reset
            myFileObj = Translate(fileobj,mode=mode,rfunc=rfunc,wfunc=wfunc)
            myFileObj = FixedBlockSize(myFileObj,cipher.block_size,mode=mode)
            #  To allow writes with seeks, we need to buffer.
            #  TODO: find a way around this.
            if mode is None:
                try:
                    mode = fileobj.mode
                except AttributeError:
                    mode = "r+"
            if self._check_mode("rw",mode):
                myFileObj = FlushableBuffer(myFileObj,mode=mode)
            elif self._check_mode("w",mode) and "-" not in mode:
                myFileObj = FlushableBuffer(myFileObj,mode=mode)
        super(Encrypt,self).__init__(myFileObj,mode=mode)




# filelike/wrappers/slice.py
#
# Copyright (C) 2006-2008, Ryan Kelly
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
#
"""

    filelike.wrappers.slice:  read/write to a slice of a file
    
This module provides the filelike wrapper 'Slice' which exposes only a
portion of a file for reading and writing.

""" 

import filelike
from filelike.wrappers import FileWrapper


class Slice(FileWrapper):
    """Class for reading/writing only a portion of a file.
    
    This file wrapper can be used to read or write to a specified
    portion of a file-like object.  The constructor argument 'start'
    indicates the offset from the beginning of the file at which
    the slice should start, and the argument 'stop' indicates the
    offset at which it should end.  For example:

        s = Slice(f,20)       # read from byte 20 onwards in f
        s = Slice(f,10,20)    # read from byte 10 up to byte 20 of f

    If 'stop' is given and the file is writable, the default behavior is
    for writes beyond the stop position to raise on IOError.  To permit
    the slice to resize itself over any existing data in the underlying
    file, specify the 'resizable' keyword argument to the constructor.

    If 'stop' is negative then it is taken as on offset from the end of the
    file, just like standard list/tuple slicing.

    """
    
    def __init__(self,fileobj,start=0,stop=None,mode=None,resizable=False):
        """Slice constuctor.

        'start' and 'stop' are the indicies at which to start and stop the
        slice, and 'resizable' indicates whether the slice is allowed to grow
        in response to writes beyond the 'stop' index.
        """
        if start < 0:
            raise ValueError("start index cannot be negative.")
        if stop is not None and stop < 0:
            try:
                stop = fileobj.size + stop
            except AttributeError:
                pos = fileobj.tell()
                fileobj.seek(0,2)
                stop = fileobj.tell() + stop
                fileobj.seek(pos,0)
        self.start = start
        self.stop = stop
        self._resizable = resizable
        super(Slice,self).__init__(fileobj,mode)
        if "a" not in self.mode:
            if self._fileobj.tell() < start:
                self._fileobj.seek(start)
    
    def _read(self,size=-1):
        """Read approximately <size> bytes from the file."""
        pos = self._fileobj.tell()
        if self.stop is not None:
            if size < 0:
                size = self.stop - pos
            elif pos + size > self.stop:
                size = self.stop - pos
        if size == 0:
            return None
        data = self._fileobj.read(size)
        if data == "":
            return None
        return data

    def _write(self,data,flushing=False):
        """Write the given string to the file."""
        if self.stop is None:
            self._fileobj.write(data)
        else:
            pos = self._fileobj.tell()
            end = pos + len(data)
            if end > self.stop:
                if self._resizable:
                    self.stop = end
                    self._fileobj.write(data)
                else:
                    self._fileobj.write(data[:(self.stop - pos)])
                    raise IOError("File not resizable")
            else:
                self._fileobj.write(data)

    def _seek(self,offset,whence):
        """Seek within the file."""
        if whence == 0:
            offset = offset + self.start
            if offset < self.start:
                offset = self.start
            if self.stop is not None:
                if offset > self.stop:
                    if self._resizable:
                        self.stop = offset
                    else:
                        offset = self.stop
            self._fileobj.seek(offset,0)
        elif whence == 1:
            pos = self._fileobj.tell()
            if pos + offset < self.start:
                offset = self.start - pos
            if self.stop is not None:
                if pos + offset > self.stop:
                    offset = self.stop - pos
            self._fileobj.seek(offset,1)
        elif whence == 2:
            if self.stop is None:
                self._fileobj.seek(offset,2)
                if offset < 0:
                    pos = self._fileobj.tell()
                    if pos < self.start:
                        self._fileobj.seek(self.start,0)
            else:
                if offset > 0 and not self._resizable:
                    offset = self.stop - self.start
                else:
                    offset = self.stop + offset - self.start
                self._seek(offset,0)
        else:
            raise ValueError("Invalid value for whence: " + str(whence))

    def _tell(self):
        """Get position of file pointer."""
        return self._fileobj.tell() - self.start

    def _truncate(self,size):
        msg = "File slices are not truncatable"
        raise filelike.NotTruncatableError(msg)
 




from filelike.wrappers import BZip2, UnBZip2
from filelike import tests
from filelike.wrappers.tests.test_buffer import get_buffered_value, def_getvalue_maybe_buffered

import unittest
from StringIO import StringIO

import bz2


class Test_BZip2(tests.Test_ReadWriteSeek):
    """Tetcases for BZip2 wrapper class."""

    contents = bz2.compress("This is my compressed\n test data")
    empty_contents = bz2.compress("")

    def makeFile(self,contents,mode):
        s = StringIO(bz2.decompress(contents))
        f = BZip2(s,mode)
        f.getvalue = def_getvalue_maybe_buffered(f,s,bz2.compress)
        return f

    #  We can't just write arbitrary text into a BZip stream, so we have
    #  to adjust these tests

    def test_write_read(self):
        self.file.write(self.contents[0:5])
        c = self.file.read()
        self.assertEquals(c,self.contents[5:])

    def test_read_write_read(self):
        c = self.file.read(5)
        self.assertEquals(c,self.contents[:5])
        self.file.write(self.contents[5:10])
        c = self.file.read(5)
        self.assertEquals(c,self.contents[10:15])

    def test_read_write_seek(self):
        c = self.file.read(5)
        self.assertEquals(c,self.contents[:5])
        self.file.write(self.contents[5:10])
        self.file.seek(0)
        c = self.file.read(10)
        self.assertEquals(c,self.contents[:10])

    def test_resulting_file(self):
        """Make sure BZip2 changes are pushed through to actual file."""
        import tempfile
        import os
        (fd,fn) = tempfile.mkstemp()
        os.close(fd)
        try:
            f = open(fn,"w")
            f.write("hello world!")
            f.close()
            f = BZip2(open(fn,"r+"))
            f.read(6)
            f.seek(-6,1)
            f.write(bz2.compress("hello Australia!"))
            f.close()
            self.assertEquals(open(fn).read(),"hello Australia!")
        finally:
            os.unlink(fn)


class Test_UnBZip2(tests.Test_ReadWrite):
    """Tetcases for UnBZip2 wrapper class."""

    contents = "This is my uncompressed\n test data"

    def makeFile(self,contents,mode):
        s = StringIO(bz2.compress(contents))
        f = UnBZip2(s,mode)
        f.getvalue = def_getvalue_maybe_buffered(f,s,bz2.decompress)
        return f

    def test_resulting_file(self):
        """Make sure UnBZip2 changes are pushed through to actual file."""
        import tempfile
        import os
        (fd,fn) = tempfile.mkstemp()
        os.close(fd)
        try:
            f = open(fn,"w")
            f.write(bz2.compress("hello world!"))
            f.close()
            f = UnBZip2(open(fn,"r+"))
            f.read(6)
            f.write("Ausralia!")
            f.seek(-6,1)
            f.write("tralia!")
            f.close()
            self.assertEquals(open(fn).read(),bz2.compress("hello Australia!"))
        finally:
          os.unlink(fn)

